<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Account
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Account_Model_Customer extends Ecart_Db_Table
{
    protected $_name = 'account_customer';
    protected $_rowClass = 'Ecart_Account_Model_Customer_Row';
    protected $_dependentTables = array('Ecart_Account_Model_Customer_Detail');
	
    /**
     * Register new customer. This method doesn't check fields for errors.
     * 
     * @param string $email
     * @param string $password
     * @param array $fields
     * @return Ecart_Account_Model_Customer Provides fluent interface
     */
    public function add($email, $password, $fields = array())
    {
        $customerId = $this->insert(array(
            'email' => $email,
            'password' => md5($password),
            'firstname' => $fields['firstname'],
            'lastname' => $fields['lastname'],
            'site_id' => Ecart::getSiteId(),
            'is_active' => 1,
            'created_at' => Ecart_Date::now()->toZendDbExpr(),
            'modified_at' => '0000-00-00 00:00:00'
        ));
        
        Ecart::single('account/customer_detail')->save(
            $customerId, $fields
        );
        
        $customer = $this->find($customerId)->current();
        Ecart::dispatch('account_customer_register_success', array(
            'customer' => $customer
        ));

        $mail = new Ecart_Mail();
        $isC = $mail->setConfig(array(
            'event'   => 'account_new-customer',
            'subject' => Ecart::translate('account')->__(
                "Welcome, %s %s",
                $fields['firstname'],
                $fields['lastname']
            ),
            'data'    => array(
                'firstname' => $fields['firstname'],
                'lastname'  => $fields['lastname'],
                'customer'  => array('email' => $email, 'password' => $password)
            ),
            'to'   => $email,
            'type' => 'html'   
        ));
        $isSend = $isC ? $mail->send() : false;
        if (!$isSend) {
            Ecart::message()->addWarning(Ecart::translate('account')->__(
                'Sorry but we were not able to contact you via email address provided.'
            ));
        }
        $mailNotice = new Ecart_Mail();
        $isC = $mailNotice->setConfig(array(
            'event'   => 'account_new-owner',
            'subject' => 'Created new account',
            'data'    => array('customer' => $customer->toArray()),
            'to'      => Ecart_Collect_MailBoxes::getName(
                Ecart::config()->orders->main->email
            ),
            'type'    => 'html'   
        ));
        if ($isC) {
            $mailNotice->send(null, false);
        }
        return $this;
    }
    
    /**
     * 
     * @param int $customerId
     * @param array $data
     * @return bool
     */
    public function save($customerId, $data)
    {
        if (!$customer = $this->find($customerId)->current()) {
            Ecart::message()->addError(Ecart::translate('account')->__(
                "Can't save customer"
            ));
            return false;
        }
        
        $clearData = array_merge(
            $data,
            array(
                'site_id' => $customer->site_id,
                'is_active' => $customer->is_active,
                'default_shipping_address_id' =>
                    $customer->default_shipping_address_id,
                'default_billing_address_id' =>
                    $customer->default_billing_address_id,
                'created_at' => $customer->created_at,
                'group_id' => $customer->group_id
            )
        );
        
        if (isset($clearData['password'])) {
            $clearData['password'] = md5($clearData['password']);
        }
        
        $customer->setFromArray($clearData);
        $customer->modified_at = Ecart_Date::now()->toZendDbExpr();
        $customer->save();
        
        Ecart::single('account/customer_detail')->save($customerId, $data);
        
        return true;
    }
    
    /**
     * 
     * @param string $email
     * @param string $password
     * @return Ecart_Account_Model_Customer Provides fluent interface
     */
    public function login($email, $password)
    {
        if (empty($email) || empty($password)) {
            return $this;
        }
        
        $auth = Zend_Auth::getInstance();
        $authAdapter = new Ecart_Auth_FrontAdapter($email, $password);
        $result = $auth->authenticate($authAdapter);

        if (!$result->isValid()) {
            Ecart::dispatch('account_customer_login_failed', array('username' => $email));
            foreach ($result->getMessages() as $error) {
                Ecart::message()->addError($error);
            }
        } else {
            Ecart::dispatch('account_customer_login_success', array('username' => $email));
            Ecart::single('checkout/cart')->merge();
            Ecart::single('checkout/checkout')->getStorage()->asGuest = null;
        }
        
        return $this;
    }
    
    /**
     * Clears private data
     * 
     * @return Ecart_Account_Model_Customer Provides fluent interface
     */
    public function logout()
    {
        Zend_Auth::getInstance()->clearIdentity();
        Ecart::single('checkout/cart')->unsetCartId();
        Ecart::single('checkout/checkout')->getStorage()->unsetAll();
        return $this;
    }

    /**
     *
     * @param array $params
     * @return array
     */
    public function getList($params = array())
    {
        $select = $this->getAdapter()->select();
        $select->from(
            array('c' => $this->_name),
            array(new Zend_Db_Expr('SQL_CALC_FOUND_ROWS DISTINCT c.id'), 'c.*')
        );
        if (isset($params['getSitesName'])) {
            $select->joinLeft(
                array('s' => self::$_db_prefix . 'core_site'),
                's.id = c.site_id',
                array('site_name' => 'name')
            );
        }
        if (isset($params['customer_email'])) {
            $select->where("c.email LIKE '%{$params['customer_email']}%'");
        }
        if (!empty($params['sort'])) {
            $select->order($params['sort'] . ' ' . $params['dir']);
        }
        if (!empty($params['limit'])) {
            $select->limit($params['limit'], $params['start']);
        }
        return array(
            'accounts' => $select->query()->fetchAll(),
            'count' => Ecart::db()->fetchOne('SELECT FOUND_ROWS()')
        );
    }

    /**
     *
     * @param int $period
     * @param string| $where
     * @return array
     */
    public function getCountList($period = 10, $where = null)
    {
        $select = $this->getAdapter()->select();
        $select->from(
                array('o' => $this->_name),
                array("LEFT(created_at, {$period}) AS pdate" ,'COUNT(*) as hit')
            )
           ->group('pdate')
           ->order('pdate ASC');
        if (is_string($where) && $where) {
            $select->where($where);
        } elseif (is_array($where)) {
            foreach ($where as $condition) {
                if ($condition) {
                    $select->where($condition);
                }
            }
        }
        return $this->getAdapter()->fetchPairs($select->__toString());
    }
    
    /**
     * Checks, is user exists in database. If not - clearIdentity called
     * 
     * @return void
     */
    public function checkIdentity()
    {
        if (!Ecart::getCustomerId()) {
            return;
        }
        if (!$this->find(Ecart::getCustomerId())->current()) {
            $this->logout();
        }
    }

    /**
     *
     * @param int $customerId
     * @return int|null
     */
    public function getGroupId($customerId)
    {
        $customerGroupId = null;

        if ($customerId) {
            $customerGroupId = $this->find($customerId)->current()->group_id;//parent::getGroupId()
        }

        if (null === $customerGroupId)  {
            $customerGroupId =  Ecart::single('account/customer_group')
                ->getIdByName('Guest');
        }
        return $customerGroupId;
    }

    /**
     * Return first create account date
     *
     * @param int $siteId
     * @return string
     */
    public function getFirstDate($siteId = null)
    {
        $select = "SELECT MIN(created_at) FROM `{$this->_name}`";
        if (null != $siteId) {
            $select .= " WHERE (site_id = '{$siteId}')";
        }
        return $this->getAdapter()->fetchOne($select);
    }

    /**
     * Return last create account date
     *
     * @param int $siteId
     * @return string
     */
    public function getLastDate($siteId = null)
    {
        $select = "SELECT MAX(created_at) FROM `{$this->_name}`";
        if (null != $siteId) {
            $select .= " WHERE (site_id = '{$siteId}')";
        }
        return $this->getAdapter()->fetchOne($select);
    }
}