<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product_Manufacturer extends Ecart_Db_Table
{
    protected $_name = 'catalog_product_manufacturer';
    
    protected $_dependentTables = array(
        'Ecart_Catalog_Model_Product_Manufacturer_Title'
    );
    
    /**
     * Retrieve list of manufacturers
     * @return array
     */
    public function getList()
    {
        return $this->getAdapter()->fetchAll("
            SELECT cpm.*, cpmt.*, ch.key_word as url
            FROM " . $this->_name . " AS cpm
            INNER JOIN " . self::$_db_prefix . "catalog_product_manufacturer_title AS cpmt
                ON cpm.id = cpmt.manufacturer_id AND language_id = " . Ecart_Locale::getLanguageId() . "
            LEFT JOIN " . self::$_db_prefix . "catalog_hurl AS ch
                ON ch.key_type = 'm' AND ch.key_id = cpm.id
            ORDER BY cpmt.title ASC
        ");
    }
    
    /**
     * Get list of manufacturers on all available languages
     * @return array
     */
    public function getListBackend()
    {
        return $this->getAdapter()->fetchAll("
            SELECT cpm.*, cpmt.*, ch.key_word as url
            FROM " . $this->_name . " AS cpm
            LEFT JOIN " . self::$_db_prefix . "catalog_product_manufacturer_title AS cpmt
                ON cpm.id = cpmt.manufacturer_id
            LEFT JOIN " . self::$_db_prefix . "catalog_hurl AS ch
                ON ch.key_type = 'm' AND ch.key_id = cpm.id
            ORDER BY cpm.id DESC
        ");
    }
    
    /**
     * @param array $data (0 => array(row_data), 1 => )
     * @return boolean
     */
    public function save($data)
    {
        if (!is_array(current($data))) {
            $success = $this->saveRow($data);
        } else {
            $success = true;
            foreach($data as $row) {
                $success = $success ? $this->saveRow($row) : false;
            }
        }
        if ($success) {
            Ecart::message()->addSuccess(
                Ecart::translate('catalog')->__(
                    'Data was successfully saved'
                )
            );
        }
        return $success;
    }
    
    /**
     * Update or delete manufacturer row
     * Checks is recieved url has duplicate before save. 
     * If it has - return false
     * 
     * @param array $row
     * @return bool
     */
    public function saveRow($row)
    {
        $manufacturer = false;
        if (isset($row['id']) && !empty($row['id'])) {
            $manufacturer = $this->find($row['id'])->current();
        }
        
        $url = trim($row['url']);
        if (empty($url)) {
            $url = strtolower($row['name']);
        }
        $url = preg_replace('/[^a-zA-Z0-9]/', '-', $url);
        if (Ecart::single('catalog/hurl')->hasDuplicate(
                $url, 
                array_keys(Ecart_Collect_Site::collect()), 
                $manufacturer ? $manufacturer->id : null
            )) {
            
            Ecart::message()->addError(
                Ecart::translate('catalog')->__(
                    'Duplicate entry (url)'
                )
            );
            return false;
        }
        
        if (!$manufacturer) {
            unset($row['id']);
            $manufacturer = $this->createRow();
        }
        $row['image'] = empty($row['image']) ? '' : '/' . trim($row['image'], '/');
        $manufacturer->setFromArray($row);
        if (false === $manufacturer->save()) {
            return false;
        }
        
        $success = true;
        
        // title
        Ecart::single('catalog/product_manufacturer_title')->delete(
            'manufacturer_id = ' . $manufacturer->id
        );
        foreach (Ecart_Collect_Language::collect() as $id => $lang) {
            $success = $success ? (bool) Ecart::single('catalog/product_manufacturer_title')->insert(array(
                'manufacturer_id' => $manufacturer->id,
                'language_id' => $id,
                'title' => isset($row['title_' . $id]) ? 
                    $row['title_' . $id] : ''
            )) : false;
        }
        
        // url
        foreach (Ecart_Collect_Site::collect() as $id => $name) {
            $success = $success ? (bool) Ecart::single('catalog/hurl')->save(
                array(
                    'site_id' => $id,
                    'key_id' => $manufacturer->id,
                    'key_type' => 'm',
                    'key_word' => $url
            )) : false;
        }
        
        return $success;
    }
    
    public function deleteByIds($ids)
    {
        if (!is_array($ids)) {
            $ids = array($ids);
        }
        $success = (bool) $this->delete(
            Ecart::db()->quoteInto('id IN(?)', $ids)
        );
        if (!$success) {
            return false;
        }
        Ecart::message()->addSuccess(
            Ecart::translate('catalog')->__(
                'Manufacturer was deleted successfully'
            )
        );
        Ecart::single('catalog/hurl')->delete(
            "key_type = 'm' AND "
            .  Ecart::db()->quoteInto('key_id IN (?)', $ids)
        );
        return $success;
    }
}