<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product_Row extends Ecart_Db_Table_Row
{
    public function save()
    {
        if (empty($this->created_on)) {
            $this->created_on = Ecart_Date::now()->toZendDbExpr();
        }
        $this->modified_on = Ecart_Date::now()->toZendDbExpr();
        if (empty($this->date_available)) {
            $this->date_available = new Zend_Db_Expr('NULL');
        }
        if (empty($this->manufacturer_id)) {
            $this->manufacturer_id = new Zend_Db_Expr('NULL');
        }
        if (empty($this->ordered)) {
            $this->ordered = 0;
        }
        if (empty($this->viewed)) {
            $this->viewed = 0;
        }
        if (empty($this->tax_class_id)) {
            $this->tax_class_id = new Zend_Db_Expr('NULL');
        }
        
        return parent::save();
    }
    
    /**
     * Updates product to category assignments
     * 
     * @param array $category
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setCategoryAssignments($category = null)
    {
        Ecart::single('catalog/product_category')->delete(
            $this->getAdapter()->quoteInto('product_id IN(?)', $this->id)
        );
        if (!is_array($category)) {
            $category[] = Ecart::single('catalog/category')->getRoot()->id;
        }
        foreach ($category as $id) {
            Ecart::single('catalog/product_category')->insert(array(
                'category_id' => $id,
                'product_id' => $this->id
            ));
        }
        return $this;
    }
    
    /**
     * Update featured status of product
     * 
     * @param bool $flag
     * @param array $siteIds
     * @param date $dateExp
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setFeatured($flag = true, $siteIds = null, $dateExp = null)
    {
        if (!is_array($siteIds)) {
            $siteIds = array(Ecart::getSiteId());
        }
        
        $tableFeatured = Ecart::single('catalog/product_featured');
        if (!$flag) {
            $tableFeatured->delete($this->getAdapter()->quoteInto(
                "product_id = {$this->id} AND site_id IN (?)",
                implode(',', $siteIds)
            ));
        } else {
            foreach ($siteIds as $siteId) {
                if (!$row = $tableFeatured->find($this->id, $siteId)->current()) {
                    $row = $tableFeatured->createRow(array(
                        'product_id' => $this->id,
                        'site_id' => $siteId
                    ));
                }
                $row->date_exp = empty($dateExp) ? new Zend_Db_Expr('NULL'): $dateExp;
                $row->save();
            }
        }
        return $this;
    }
    
    /**
     * Update product_stock table 
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setStock($data = null)
    {
        if (null === $data) {
            return $this;
        }
        
        $stock = Ecart::single('catalog/product_stock');
        
        if (!$row = $stock->find($this->id)->current()) {
            $row = $stock->createRow();
            $row->product_id = $this->id;
        }
        
        foreach ($data as $col => $val) {
            if (empty($val)) {
                $data[$col] = intval($data[$col]);
            }
        }
        $oldBackorder = $row->backorder;
        $oldInStock = $row->in_stock;
        $row->setFromArray($data);
        $row->save();
        Ecart::dispatch('catalog_product_update_stock', array(
            'product' => $this,
            'stock' => $row,
            'old_backorder' => $oldBackorder,
            'old_in_stock' => $oldInStock
        ));
        return $this;
    }
    
    /**
     * Update product images
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setImage($data = null)
    {
        $gallery = Ecart::single('catalog/product_gallery');
        
        $gallery->delete(
            $this->getAdapter()->quoteInto('product_id = ? ', $this->id)
        );
        
        if (null === $data) {
            return $this;
        }
        
        foreach ($data as $imageId => $imagePath) {
            $gallery->insert(array(
                'product_id' => $this->id,
                'image' => $imagePath
            ));
        }
        return $this;
    }
    
    /**
     * Update discount on product
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setSpecial($data)
    {
        if(!empty($data['from_date_exp'])) {
            $data['from_date_exp'] .= ' 00:00:00';
        }
        if(!empty($data['to_date_exp'])) {
            $data['to_date_exp'] .= ' 23:59:59';
        }
        if (!empty($data['price'])) {
            Ecart::single('discount/discount')->setSpecialPrice(
                $this->id, $data['price'],
                $data['from_date_exp'],
                $data['to_date_exp']  
            );
        }
        return $this;
    }
    
    /**
     * Update new product status
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setNew($data)
    {
        $this->new_from = empty($data['from_date']) ? new Zend_Db_Expr('NULL') : $data['from_date'];
        $this->new_to = empty($data['to_date']) ? new Zend_Db_Expr('NULL') : $data['to_date'];
        $this->save();
        return $this;
    }
    
    /**
     * Updates product description
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setDescription($data)
    {
        $tableDesc = Ecart::single('catalog/product_description');
        foreach ($data as $langId => $values) {
            if (!$row = $tableDesc->find($this->id, $langId)->current()) {
                $row = $tableDesc->createRow();
                $row->product_id = $this->id;
                $row->language_id = $langId;
            }
            $row->setFromArray($values);
            $row->save();
        }
        return $this;
    }
    
    /**
     * Update variations
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setVariation($data)
    {
        $mVariation = Ecart::single('catalog/product_variation');
        $mAttribute = Ecart::single('catalog/product_attribute');
        foreach ($data as $variation) {
            if (!is_numeric($variation['id']) 
                || !$row = $mVariation->find($variation['id'])->current()) {

                unset($variation['id']);
                $row = $mVariation->createRow();
                $row->product_id = $this->id;
            }
            
            if ($variation['status'] == 'removed') {
                $row->delete();
            } else {
                $row->setFromArray($variation);
                $row->save();
                
                foreach ($variation['options'] as $option) {
                    if ($option['status'] == 'new') {
                        $mAttribute->insert(array(
                            'product_variation_id' => $row->id,
                            'product_id' => $this->id,
                            'product_option_id' => $option['product_option_id'],
                            'product_option_value_id' =>
                                isset($option['product_option_value_id']) ?
                                    $option['product_option_value_id'] :
                                    new Zend_Db_Expr('NULL')
                        ));
                    } elseif ($option['status'] == 'removed') {
                        $mAttribute->delete('id = ' . intval($option['id']));
                    }
                }
            }
        }
        return $this;
    }
    
    /**
     * Update modifiers
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setModifier($data)
    {
        $modelAttribute = Ecart::single('catalog/product_attribute');
        foreach ($data as $modifier) {
            if (!isset($modifier['id']) || 
                !is_numeric($modifier['id']) || 
                !$row = $modelAttribute->find($modifier['id'])->current())
            {
                $row = $modelAttribute->createRow();
                $row->product_id = $this->id;
                $row->modifier = 1;
            }
            
            if ($modifier['status'] == 'removed') {
                $row->delete();
            } else {
                $row->setFromArray($modifier);
                $row->save();
            }
        }
        return $this;
    }
    
    /**
     * Update properties
     * 
     * @param array $data
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setProperty($data)
    {
        $tableAttribute = Ecart::single('catalog/product_attribute');
        $tableAttrValue = Ecart::single('catalog/product_attribute_value');
        foreach ($data as $property) {
            if (!isset($property['id']) ||
                !is_numeric($property['id']) ||
                !$row = $tableAttribute->find($property['id'])->current())
            {
                unset($property['id']);
                $row = $tableAttribute->createRow();
                $row->product_id = $this->id;
            }
            
            if ($property['status'] == 'removed') {
                $row->delete();
            } else {
                $row->setFromArray($property);
                $row->save();
                
                if (isset($property['values'])) {
                    foreach (Ecart_Collect_Language::collect() as 
                        $languageId => $language) {

                        if (!isset($property['values'][$languageId])) {
                            continue;
                        }
                        if (!$valus_row = $tableAttrValue
                            ->find($row->id, $languageId)->current()) {


                            $valus_row = $tableAttrValue->createRow();
                            $valus_row->product_attribute_id = $row->id;
                            $valus_row->language_id = $languageId;
                        }
                        $valus_row->attribute_value =
                            $property['values'][$languageId];
                        $valus_row->save();
                    }
                }
            }
        }
        return $this;
    }
    
    /**
     * Update human url
     * 
     * @param string $url
     * @param array $siteIds
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function setUrl($url = '', $siteIds = null)
    {
        $url = trim($url);
        if (empty($url)) {
            $description = $this->getDescription(Ecart::getLanguageId());
            $url = strtolower($description['name']); 
        }
        
        if (empty($url)) {
            Ecart::message()->addError(
                Ecart::translate('catalog')->__(
                    'Please fill product url field'
                )
            );
            return $this;
        }
        
        $url = str_replace(
            array('\\', '/', ' '),
            Ecart::config('catalog/product/hurldelimiter', '-'),
            $url
        );
        
        if (!is_array($siteIds)) {
            $siteIds[] = Ecart::getSiteId();
        }
        
        $tableHurl = Ecart::single('catalog/hurl');
        
        if ($tableHurl->hasDuplicate($url, $siteIds, $this->id)) {
            Ecart::message()->addError(
                Ecart::translate('catalog')->__(
                    'Product url not saved. Duplicate entry url'
                )
            );
            return $this;
        }
        
        foreach ($siteIds as $siteId) {
            $where = array(
                'site_id = ' . intval($siteId),
                'key_id = '  . $this->id,
                "key_type = 'p'" 
            );
            if (!$row = $tableHurl->fetchRow($where)) {
                $row = $tableHurl->createRow();
                $row->site_id = $siteId;
                $row->key_type = 'p';
                $row->key_id = $this->id;
            }
            $row->key_word = $url;
            $row->save();
        }
        
        return $this;
    }
    
    /**
     * Retrieve array of category paths, where the product lies in
     * Array(
     *     // path 1
     *     [0] => Array(
     *          Array (
     *             [lvl] => 1
     *             [key_word] => modern-style
     *             [id] => 59
     *             [name] => Modern Style
     *          )
     *          Array (
     *             [lvl] => 2
     *             [key_word] => inner-modern-style
     *             [id] => 60
     *             [name] => Inner Modern Style
     *          )
     *     )
     *     // path 2
     *     [1] => ...
     * )
     * 
     * @param integer $catId defines which of product paths is more important (if product has 2 or more paths)
     * @param boolean $fullSet if true - function will return all paths of product
     * @return array One of founded paths
     */
    public function getParentItems($catId = null, $fullSet = false)
    {
        $siteId = Ecart::getSiteId();
        $select = $this->getAdapter()->select();
        $select->from(array('ptc' => self::$_db_prefix . 'catalog_product_category'), array())
            ->joinLeft(array('c1' => self::$_db_prefix . 'catalog_category'), 'c1.id = ptc.category_id', array())
            ->joinLeft(array('c2' => self::$_db_prefix . 'catalog_category'), 'c1.lft BETWEEN c2.lft AND c2.rgt',
                array('lvl'))
            ->join(array('hu' => self::$_db_prefix . 'catalog_hurl'), 'hu.key_id = c2.id', array('key_word'))
            ->join(array('cd' => self::$_db_prefix . 'catalog_category_description'),
                'cd.category_id = c2.id',
                array('id' => 'category_id', 'name', 'meta_title', 'meta_description', 'meta_keyword'))
            ->where($this->getAdapter()->quoteInto('ptc.product_id = ?', $this->id))
            ->where('c1.site_id = ' . $siteId)
            ->where('c2.site_id = ' . $siteId)
            ->where('cd.language_id = ' . Ecart::getLanguageId())
            ->where("hu.key_type = 'c'")
            ->order(array('ptc.category_id', 'ptc.product_id', 'c2.lvl'));
        
        $paths = $select->query()->fetchAll();
        $result = array();
        $i = $prev_lvl = 0;
        foreach ($paths as $path) {
            if ($prev_lvl >= $path['lvl']) {
                $i++;
            }
            $result[$i][] = $path;
            $prev_lvl = $path['lvl'];
        }
        
        if ($fullSet) {
            return $result;
        }
        
        /* find most suitable path */
        if (null !== $catId) {
            foreach ($result as $path) {
                foreach($path as $category) {
                    if (in_array($catId, $category))
                        return $path;
                }
            }
        }
        
        return current($result) ? current($result) : array(); 
    }
    
    /**
     * Check is product available to buy without variation.
     * 
     * @return bool
     */
    public function isAvailable()
    {
        $stock = Ecart::single('catalog/product_stock')->find($this->id)->current();
        
        if (!$stock->in_stock) {
            return false;
        }
        
        if (($stock->min_qty_allowed > $this->quantity || 
            $stock->min_qty >= $this->quantity) && !$stock->backorder) {
            return false;
        }

        return true;
    }
    
    /** 
     * Check is product available to buy without or with variation
     * 
     * @return bool
     */
    public function isSaleable()
    {
        if ($this->isAvailable()) {
            return true;
        }
        
        $variations = $this->getVariations();
        foreach ($variations as $variation) {
            if ($variation['quantity'] > 0) {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Retrieve description row for product
     * 
     * @param int $languageId
     * @return array
     */
    public function getDescription($languageId = null)
    {   
        if (null === $languageId) {
            $languageId = Ecart::getLanguageId();
        }
        $query = "SELECT * FROM " . self::$_db_prefix. "catalog_product_description " . 
            "WHERE product_id = ? AND language_id = ?";
        
        return $this->getAdapter()->fetchRow($query, array($this->id, $languageId));
    }
    
    public function getDescriptions()
    {
        return $this->findDependentRowset('Ecart_Catalog_Model_Product_Description');
    }
    
    public function getProperties()
    {
        $query = "SELECT pot.name as name, if (povt.name !='', povt.name, pav.attribute_value) as value
		FROM " . self::$_db_prefix . "catalog_product_attribute pa
		INNER JOIN " . self::$_db_prefix . "catalog_product_option po ON po.id = pa.product_option_id
		INNER JOIN " . self::$_db_prefix . "catalog_product_option_text pot ON pot.product_option_id = pa.product_option_id
		LEFT JOIN  " . self::$_db_prefix . "catalog_product_option_value_text povt ON povt.product_option_value_id = pa.product_option_value_id AND " 
												   ."povt.language_id = :langId
		LEFT JOIN " . self::$_db_prefix . "catalog_product_attribute_value pav ON pav.product_attribute_id = pa.id AND " 
												. "(pav.language_id = :langId OR pav.language_id is null)
		WHERE pa.product_id = :prodId AND pa.product_variation_id = 0 AND pa.modifier = 0 AND
		  pot.language_id = :langId
		ORDER BY po.sort_order, value";
        
        $ret =  $this->getAdapter()->fetchAll($query, array(
            'langId' => Ecart::getLanguageId(),
            'prodId' => $this->id
        ));
        
        return $ret;
    }
    
    public function getModifiersFull()
    {
        $query = "SELECT pa.*, pot.name as option_name, povt.name as value_name, po.option_type
		FROM " . self::$_db_prefix. "catalog_product_attribute pa
		INNER JOIN " . self::$_db_prefix . "catalog_product_option po ON po.id = pa.product_option_id
		INNER JOIN " . self::$_db_prefix . "catalog_product_option_text pot ON pot.product_option_id = pa.product_option_id
		LEFT JOIN " . self::$_db_prefix . "catalog_product_option_value pov ON pov.id = pa.product_option_value_id
		LEFT JOIN " . self::$_db_prefix . "catalog_product_option_value_text povt ON povt.product_option_value_id = pa.product_option_value_id AND povt.language_id = :langId
		WHERE pa.product_id = :prodId AND
		      pa.modifier = 1 AND
		      pot.language_id = :langId
		ORDER BY po.sort_order, pov.sort_order, value_name";
        $rows = $this->getAdapter()->fetchAll($query, array(
            'langId' => Ecart::getLanguageId(),
            'prodId' => $this->id
        ));
        
        $modifiers = array();
        foreach ($rows as $row) {
            $oId = $row['product_option_id'];
            if (!isset($modifiers[$oId]))
                $modifiers[$oId] = array(
                    'id' => $oId,
                    'name' => $row['option_name'],
                    'type' => $row['option_type'],
                    'attribute_id' => $row['id'],
                    'values' => array()
                );
            $modifiers[$oId]['values'][] = array(
                'id' => $row['product_option_value_id'],
                'name' => $row['value_name'],
                'attribute_id' => $row['id'],
                'price' => $row['price'],
                'price_type' => $row['price_type']
            );
        }
        return $modifiers;
    }
    
    public function getModifiers()
    {
        $query = "SELECT pa.*, po.option_type
        FROM " . self::$_db_prefix. "catalog_product_attribute pa
        INNER JOIN " . self::$_db_prefix . "catalog_product_option po ON po.id = pa.product_option_id
        LEFT JOIN " . self::$_db_prefix . "catalog_product_option_value pov ON pov.id = pa.product_option_value_id
        WHERE pa.product_id = :prodId AND
              pa.modifier = 1";
        $rows = $this->getAdapter()->fetchAll($query, array(
            'prodId' => $this->id
        ));
        
        $modifiers = array();
        foreach ($rows as $row) {
            $oId = $row['product_option_id'];
            if (!isset($modifiers[$oId]))
                $modifiers[$oId] = array(
                    'id' => $oId,
                    'type' => $row['option_type'],
                    'attribute_id' => $row['id'],
                    'values' => array()
                );
            $modifiers[$oId]['values'][] = array(
                'id' => $row['product_option_value_id'],
                'attribute_id' => $row['id'],
                'price' => $row['price'],
                'price_type' => $row['price_type']
            );
        }
        return $modifiers;
    }
    
    public function getVariations()
    {
        return $this->findDependentRowset('Ecart_Catalog_Model_Product_Variation');
    }
    
    public function getVariationAttributesData()
    {
        $query = 'SELECT pa.*, pot.name as option_name, povt.name as value_name, po.option_type
            FROM ' . self::$_db_prefix. 'catalog_product_attribute pa
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_option po ON po.id = pa.product_option_id
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_option_text pot ON pot.product_option_id = pa.product_option_id
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_option_value pov ON pov.id = pa.product_option_value_id
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_option_value_text povt ON povt.product_option_value_id = pa.product_option_value_id
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_variation AS cpv ON pa.product_variation_id = cpv.id
            INNER JOIN ' . self::$_db_prefix. 'catalog_product_stock AS cps ON pa.product_id = cps.product_id AND
            ((cps.backorder = 0 AND cpv.quantity > 0) OR (cps.backorder > 0))
            WHERE pa.product_id = :prodId AND
                      pa.product_variation_id > 0 AND
                      pot.language_id = :langId AND povt.language_id = :langId
            ORDER BY po.sort_order, pov.sort_order, pa.id';
        
        $res = $this->getAdapter()->query($query, array(
            'prodId' => $this->id,
            'langId' => Ecart::getLanguageId()
        ));
        $optionsLabels = array();
        $valuesLabels  = array();
        $options       = array();
        $variations    = array();
        while (($row = $res->fetch())) {
            $poId = $row['product_option_id'];
            $povId = $row['product_option_value_id'];
            
            $optionsLabels[$poId] = $row['option_name'];
            $valuesLabels[$povId] = $row['value_name'];
            $options[$poId][$povId] = $povId;
            $variations[$row['product_variation_id']][$poId] = $povId;
        }
        return array(
            'variationsAssign' => $variations,
            'optionsAssign'    => $options,
            'optionsLabels'    => $optionsLabels,
            'valuesLabels'     => $valuesLabels
        );
    }
    
    public function getPriceRules()
    {
        $price = $this->price;
        $variations = Ecart::single('catalog/product_variation')->fetchAll(
            ' product_id = ' . $this->id
        );
        $modifiers = Ecart::single('catalog/product_attribute')->fetchAll(
            ' product_id = ' . $this->id . ' AND ' . 
            ' modifier = 1 AND product_variation_id = 0'
        );

        $res = array(
            'price' => $this->price,
            'finalPrice' => $price,
            'currencyRate' => Ecart::single('locale/currency')->getData(null, 'rate')
        );
        
        if (!count($variations) && !count($modifiers)) {
            return $res;
        }
        //$difference = array();
            
        foreach ($variations as $variation) {

            $diff = self::getNewPrice($price, $variation->price, $variation->price_type);
            $diff = $diff - $price;
            $amount = $variation->price;

            //$difference[] = $diff;
            $res['variation'][$variation->id] = array(
                'id'         => $variation->id,
                'amount'     => $amount,
                'type'       => $variation->price_type,
                'difference' => $diff
            );
        }
       
        foreach ($modifiers as $modifier) {
            $diff = self::getNewPrice($price, $modifier->price, $modifier->price_type);
            
            $diff = $diff - $price;
            $amount = $modifier->price;

            //$difference[] = $diff;
            $res['modifier'][$modifier->id] = array(
                'id' => $modifier->id,
                'optionId'  => $modifier->product_option_id,
                'amount'      => $amount,
                'type' => $modifier->price_type,
                'difference' => $diff
            );
        }
        //$res['from'] = min($difference);
        //$res['to'] = max($difference);
        //Zend_Debug::dump($res);
        return $res;
    }

    
    /**
     * Высчитывает цену учитывая набор атрибутов и скидку на продукт
     * 
     * @return float
     */
    public function getPrice($attributeIds = array())
    {
        $price = $this->price;
        if (!count($attributeIds)) {
            $discountRule = Ecart::single('discount/discount')
                ->getRuleByProductId($this->id);
            $price = Ecart::single('discount/discount')->applyDiscountRule(
                $price, $discountRule
            );
            return $price;
        }
        $where = $this->getAdapter()->quoteInto('?', $attributeIds);
        $attributes = $this->getAdapter()->fetchAll(
            "SELECT pa.*, po.sort_order, pov.sort_order as sort_order2 FROM " . 
            self::$_db_prefix . "catalog_product_attribute AS pa  
            LEFT JOIN " . self::$_db_prefix . "catalog_product_option AS po ON pa.product_option_id = po.id
            LEFT JOIN " . self::$_db_prefix . "catalog_product_option_value pov ON pov.id = pa.product_option_value_id
            WHERE pa.id IN($where) ORDER BY po.sort_order ASC, pov.sort_order"
        );
        
        $variationId = 0;
        $changeStack = array();
        // (//) => TO TO Kostul`
        $priceTo = array(); //
        foreach ($attributes as $attribute) {
            if ($attribute['product_variation_id']) {
                $variationId = $attribute['product_variation_id'];
            }
            if ($attribute['modifier']) {
                
                if ($attribute['price_type'] != 'to') {//
                    
                    $changeStack[] = array(
                        'price'       => $attribute['price'],
                        'price_type'  => $attribute['price_type']
                    );
                } else {//
                    $priceTo[] = $attribute['price'];//
                }//
            }
        }
        
        // set variation
        if ($variationId) {
            $variation = Ecart::single('catalog/product_variation')->fetchRow('id = ' . $variationId)->toArray();
            $price = self::getNewPrice($price, $variation['price'], $variation['price_type']);
        }
        // set discount
        $discountRule = Ecart::single('discount/discount')->getRuleByProductId(
            $this->id, $variationId
        );
        $price = Ecart::single('discount/discount')->applyDiscountRule(
            $price, $discountRule, $attributeIds
        );
        
        // set max modifier to
        if (count($priceTo)) { //
            $price = self::getNewPrice($price, max($priceTo), 'to');//
        }//

        //apply "by" and "percent" modifier
        $modifierAmount = 0;
        foreach ($changeStack as $sort) {
            //foreach ($sort as $item) {
                $modifierAmount += - $price + self::getNewPrice($price, $sort['price'], $sort['price_type']);
            //}
        }
        $price += $modifierAmount;

        return  $price;
    }
    
    /**
     * Высчитывает вес учитывая набор атрибутов
     * 
     * @return float
     */
    public function getWeight($attributes = array())
    {
        $db = $this->getAdapter();
        
        $weight = $this->weight;
        
        if (!count($attributes)) {
            return $weight;
        }
        
        $where = $db->quoteInto('?', $attributes);
        $attributes = Ecart::single('catalog/product_attribute')->fetchAll("id IN($where)")->toArray();
        $variationId = 0;
        $changeStack = array();
        foreach ($attributes as $attribute) {
            if ($attribute['product_variation_id']) {
                $variationId = $attribute['product_variation_id'];
            }
            if ($attribute['modifier']) {
                $changeStack[] = array(
                    'weight'      => $attribute['weight'],
                    'weight_type' => $attribute['weight_type']
                );
            }
        }
        if ($variationId) {
            $variation = Ecart::single('catalog/product_variation')->fetchRow('id = ' . $variationId)->toArray();
            $weight = self::getNewWeight($weight, $variation['weight'],$variation['weight_type']);
        }
        
        foreach ($changeStack as $item) {
            $weight = self::getNewWeight($weight, $item['weight'], $item['weight_type']);
        }
        
        return $weight;
    }
    
    /**
     * Возвращает связь вариаций к ее атрибутам.
     * array(
     *     variationId => array(optionId => valueId, optionId => valueId),
     *     variationId => array(optionId => valueId, optionId => valueId)
     * )
     * 
     * @return array
     */
    public function getVariationsAssign()
    {
        $select = $this->getAdapter()->select();
        $select->from(self::$_db_prefix . 'catalog_product_attribute', array(
            'product_variation_id',
            'product_option_id',
            'product_option_value_id'
        ))
        ->where('product_id = ' . $this->id)
        ->where('product_variation_id > 0');
        $res = $select->query();
        $variations = array();
        while (($row = $res->fetch())) {
            $poId = $row['product_option_id'];
            $povId = $row['product_option_value_id'];
            
            $variations[$row['product_variation_id']][$poId] = $povId;
        }
        return $variations;
    }
    
    /**
     * Возвращает вариацию подпадающую под данный набор атрибутов
     * 
     * @param array $options options: array(optionId => valueId, optionId => valueId)
     * @return Zend_Db_Table_Row_Abstract
     */
    public function getVariationIdByAttributes($options)
    {
        $variations = $this->getVariationsAssign();
        
        $maxMatch = 0;
        $currentDiff = 0;
        $res = 0;
        foreach ($variations as $vId => $variation) {
            $match = count(array_intersect_assoc($variation, $options));
            $diff = count(array_diff_assoc($variation, $options));
            if ($maxMatch < $match) {
                $maxMatch = $match;
                $currentDiff = $diff;
                $res = $vId;
            } elseif ($maxMatch == $match) {
                if ($diff == 0) {
                    $res = $vId;                    
                } elseif ($currentDiff >= $diff) {
                    $res = 0;    
                }
            }
        }
        
        return $res;
    }
    
    public function getAttributesByVariationId($vId)
    {
        return Ecart::single('catalog/product_attribute')->fetchAll("product_variation_id = " . $vId)->toArray();
    }

    /**
     *
     * @return string
     */
    public function getHumanUrl()
    {
        return Ecart::single('catalog/hurl')->fetchOne(
            '*', "key_id = ? && key_type = ?", array($this->id, 'p')
        );
    }
    
    public function incViewed()
    {
        $productDescription = Ecart::single('catalog/product_description')
            ->find($this->id, Ecart::getLanguageId())->current();
        
        if (isset($productDescription->viewed)) {
            $productDescription->viewed++;
            $productDescription->save(); 
        }
        $this->viewed++;
        return $this->save(); 
    }
   
    public function getGallery()
    {
        return $this->findDependentRowset('Ecart_Catalog_Model_Product_Gallery');
    }
    
    public function getAttributes()
    {
        return $this->findDependentRowset('Ecart_Catalog_Model_Product_Attribute');
    }
    
    /**
     * Add one or more images to gallery
     * 
     * @param string|array $images
     */
    public function addToGallery($images)
    {
        if (is_string($images)) {
            $images = array($images);
        }
        foreach ($images as $image) {
            Ecart::single('catalog/product_gallery')->insert(array(
                'product_id' => $this->id,
                'image' => $image
            ));
        }
    }
    
    public function getStock()
    {
        return Ecart::single('catalog/product_stock')->getStock($this->id);
    }
    
    public function getQuantity($variationId = null, $availableOnly = false)
    {
        return Ecart::single('catalog/product_stock')->getQuantity($this->id, $variationId, $availableOnly);
    }
    
    public function canAddToCart($quantity, $variationId = null)    
    {
        return Ecart::single('catalog/product_stock')->canAddToCart(array(
            'product_id' => $this->id,
            'quantity'   => $quantity,
            'product_variation_id' => $variationId
        ));
    }
    
    public function getCategories()
    {
        return $this->getTable()->getCategories($this->id);
    }

    /**
     *
     * @static
     * @param float $price
     * @param float $amount
     * @param string $amountType
     * @return float
     */
    public static function getNewPrice($price, $amount, $amountType)
    {
        $price  = floatval($price);
        $amount = floatval($amount);
        if ($amount == 0) {
            return $price;
        }
        switch ($amountType) {
            case 'to':
                return $amount;
            case 'by':
                return $price + $amount;
            case 'percent':
                return $price + ($price * $amount / 100);
            default:
                return $price;
        }
    }

    /**
     *
     * @static
     * @param float $weight
     * @param float $amount
     * @param string $amountType
     * @return float
     */
    public static function getNewWeight($weight, $amount, $amountType)
    {
        $weight = floatval($weight);
        $amount = floatval($amount);
        if ($amount == 0)
            return $weight;
        switch ($amountType) {
            case 'to':
                return $amount;
            case 'by':
                return $weight + $amount;
            case 'percent':
                return $weight + ($weight * $amount / 100);
            default:
                return $weight;
        }
    }
}