<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product_Stock extends Ecart_Db_Table  
{
    protected $_name = 'catalog_product_stock';
    protected $_rowClass = 'Ecart_Catalog_Model_Product_Stock_Row';
    protected $_primary = array('product_id');
    protected $_referenceMap = array(
        'Product' => array(
            'columns'       => 'product_id',
            'refTableClass' => 'Ecart_Catalog_Model_Product',
            'refColumns'    => 'id'
        )
    );
    
    /**
     * @param int 
     * @return  array()
     */
    public function getStock($productId)
    {
        $product = Ecart::single('catalog/product')->find($productId)->current();
        $variations = Ecart::single('catalog/product_variation')->fetchAll(' product_id = ' . $productId);
        $row =  $this->find($productId)->current()->toArray();
        $row['quantity']["{$product->sku}"] = $product->quantity;
        foreach ($variations as $variation) {
            $row['quantity']["{$variation->sku}"] = $variation->quantity;
        }
        return $row;
    }
    
    /**
     * @param array $product
     * @param boolean $backorderCondition [optional]
     * @param boolean $outputMessages [optional]
     * @return boolean
     */
    protected function _available(array $product, $backorderCondition = false, $outputMessages = false) 
    {
        if (!isset($product['product_id']) || !isset($product['quantity'])) {
            return false;
        }
        
        $stock = $this->find($product['product_id'])->current();
        
        if (!$stock->in_stock) {
            if ($outputMessages) {
                Ecart::message()->addError(
                    Ecart::translate('catalog')->__(
                        'Product is out of stock'
                    )
                );
            }
            return false;
        }
        
        if (($product['quantity'] < $stock->min_qty_allowed) 
            || ($product['quantity'] <= 0)) {
            
            if ($outputMessages) {
                Ecart::message()->addError(
                    Ecart::translate('catalog')->__(
                        'Minimum allowed quantity is %d',
                        $stock->min_qty_allowed > 0 ? $stock->min_qty_allowed : 1
                    )
                );
            }
            return false;
        }
        
        if ($product['quantity'] > $stock->max_qty_allowed 
            && $stock->max_qty_allowed != 0) {
            
            if ($outputMessages) {
                Ecart::message()->addError(
                    Ecart::translate('catalog')->__(
                        'Maximum allowed quantity is %d', $stock->max_qty_allowed
                    )
                );
            }
            return false;
        }
        
        $stockQuantity = $this->getQuantity(
            $product['product_id'], 
            $product['product_variation_id']
        );
        $availableQuantity = $stockQuantity - $stock->min_qty;
        
        if ($backorderCondition 
            && $product['quantity'] > $availableQuantity 
            && !$stock->backorder) {
            
            if ($outputMessages) {
                Ecart::message()->addError(
                    Ecart::translate('catalog')->__(
                        'Only %d %s',
                        $availableQuantity,
                        $availableQuantity > 1 ?
                            Ecart::translate('catalog')->__('items are available')
                            : Ecart::translate('catalog')->__('item is available')
                    )
                );
            }
            return false;
        }
        
        if (!$backorderCondition
            && $product['quantity'] > $availableQuantity) {
            
            if ($outputMessages) {
                Ecart::message()->addError(
                    Ecart::translate('catalog')->__(
                        'Only %d %s available',
                        $availableQuantity,
                        $availableQuantity > 1 ?
                            Ecart::translate('catalog')->__('items are available')
                            : Ecart::translate('catalog')->__('item is available')
                    )
                );
            }
            return false;
        }
        return true;
    }
    
    /**
     * Checks, posibility to set pending status 
     * on a product from shopping cart
     * 
     * @param array $product
     * @return boolean
     */
    public function canPending(array $product, $outputMessages = false)
    {
        return $this->_available($product, true, $outputMessages);
    }
    
    /**
     *  @param array $product
     *  @return bool
     */
    public function canShipping(array $product, $outputMessages = false)
    {
        return $this->_available($product, false, $outputMessages);
    }
    
    /**
     *  @param $product 
     *  array(  
     *      'product_id' => 
     *      'quantity' => 
     *      'product_variation_id' =>
     *  )
     *  @return boolean
     */
    public function canAddToCart(array $product, $outputMessages = false) 
    {
        return $this->_available($product, true, $outputMessages);
    }
    
    /**
     * Retrieve product or variation quantity 
     * 
     * @param int $productId
     * @param int $variationId
     * @param bool $availableOnly TRUE to get available to buy only
     * @return mixed (int|bool) 
     */
    public function getQuantity($productId, $variationId = null, $availableOnly = false)
    {
        if (!$stock = $this->find($productId)->current()) {
            return false;
        }
        
        if ($variationId) {
            $variation = Ecart::single('catalog/product_variation')->find($variationId)->current();
            if ($variation->product_id == $productId) {
                $quantity = $variation->quantity;
            } else {
                return false;
            } 
        } else {
            $quantity = Ecart::single('catalog/product')->find($productId)->current()->quantity;
        }
        if ($availableOnly) {
            $quantity -= $stock->min_qty;
        }
        return $stock->decimal ? $quantity : floor($quantity);
    }
    
    /**
     * Update product or variation quantity. Frontend method.
     * Called at Ecart_Sales_Model_Order_Status_Run
     * 
     * @param int $productId
     * @param float $quantity
     * @param int $variationId [optional]
     * @return boolean
     */
    public function setQuantity($productId, $quantity, $variationId = null)
    {
        if (!$stock = $this->find($productId)->current()) {
            return false;
        }
        if (!$stock->manage) {
            return true;
        }
        // if backorder quantity change on ship order
        if ($stock->backorder && $quantity < $stock->min_qty) {
            return true;
        }
        if ($stock->decimal) {
            $quantity = floor($quantity);
        }
        if ($variationId) {
            $variation = Ecart::single('catalog/product_variation')->find($variationId)->current();
            if ($variation->product_id == $productId) {
                $oldQuantity = $variation->quantity;
                $variation->quantity = $quantity;
                $variation->save();
                Ecart::dispatch('catalog_product_update_quantity', array(
                    'product' => $variation,
                    'stock' => $stock,
                    'old_quantity' => $oldQuantity
                ));
                return true;
            } else {
                return false;
            } 
        }
        $product = Ecart::single('catalog/product')->find($productId)->current();
        $oldQuantity = $product->quantity;
        $product->quantity = $quantity;
        $product->save();
        Ecart::dispatch('catalog_product_update_quantity', array(
            'product' => $product,
            'stock' => $stock,
            'old_quantity' => $oldQuantity
        ));
        return true;
    }
}