<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Model_Checkout extends Ecart_Object
{
    /**
     * @var Zend_Session_Namespace
     */
    private $_storage;
    
    /**
     * @var Ecart_Checkout_Model_Total
     */
    private $_total;

    /**
     * @var Ecart_Checkout_Model_Cart
     */
    private $_cart;
    
    /**
     * @var Ecart_Shipping_Model_Method_Abstract
     */
    private $_shipping;
    
    /**
     * @var Ecart_Payment_Model_Method_Abstract
     */
    private $_payment;

    private $_errors;
    
    const ERROR_CUSTOMER  = 'unknown_customer';
    const ERROR_DELIVERY  = 'unknown_delivery';
    const ERROR_SHIPPPING = 'unknown_shipping';
    const ERROR_BILLING   = 'unknown_billing';
    const ERROR_PAYMENT   = 'unknown_payment';
    
    public function clean()
    {
        $this->getStorage()->unsetAll();    
        $this->_storage = null;
    }
    
    /**
     * Return current shopping cart
     *
     * @return Ecart_Checkout_Model_Cart
     */
    public function getCart()
    {
        if (!$this->_cart instanceof Ecart_Checkout_Model_Cart) {
            $this->_cart = Ecart::single('checkout/cart');
        }
        return $this->_cart;
    }
    
    /**
     * Return current Checkout_Total model
     *
     * @return Ecart_Checkout_Model_Total
     */
    public function getTotal()
    {
        if (!$this->_total instanceof  Ecart_Checkout_Model_Total) {
            $this->_total = Ecart::single('checkout/total');
        }
        return $this->_total;
    }

    /**
     * Return checkout data storage
     *
     * @return Zend_Session_Namespace
     */
    public function getStorage()
    {
        if (!$this->_storage instanceof Zend_Session_Namespace) {
            $this->_storage = new Zend_Session_Namespace('Checkout');
        }
        return $this->_storage;
    }
    
    /**
     * Return information, that is needed by shipping methods
     * 
     * @return array
     */
    public function getShippingRequest()
    {
        if (!Zend_Registry::isRegistered('shippingRequest')) {
            $request = (null === $this->getStorage()->delivery) ?
                array() : $this->getStorage()->delivery;

            $request['boxes']  = 1;
            $request['qty']    = $this->getCart()->getCount();
            $request['weight'] = $this->getCart()->getTotalWeight();
            $request['price']  = $this->getTotal()->getTotal('Checkout_Subtotal');
            $request['currency'] = Ecart::config()->main->store->baseCurrency;
            Zend_Registry::set('shippingRequest', $request);
        }
        return Zend_Registry::get('shippingRequest');
    }
    
    public function getPaymentRequest()
    {
        if (!Zend_Registry::isRegistered('paymentRequest')) {
            $request = (null === $this->getStorage()->billing) ?
                array() : $this->getStorage()->billing;

            $request['boxes']  = 1;
            $request['qty']    = $this->getCart()->getCount();
            $request['weight'] = $this->getCart()->getTotalWeight();
            $request['price']  = $this->getTotal()->getTotal();
            $request['currency'] = Ecart::config()->main->store->baseCurrency;
            Zend_Registry::set('paymentRequest', $request);
        }
        return Zend_Registry::get('paymentRequest');
    }
    
    /**
     * Return current shipping module instance
     *
     * @return Ecart_Shipping_Model_Method_Abstract|null
     */
    public function shipping()
    {
        if (null === $this->_shipping && isset($this->getStorage()->shipping)) {
            $this->_shipping = Ecart_Shipping::getMethod($this->getStorage()->shipping);
        }
        return $this->_shipping;
    }
    
    /**
     * Return current payment module instance
     *
     * @return Ecart_Payment_Model_Method_Abstract|null
     */
    public function payment()
    {
        if (null === $this->_payment && isset($this->getStorage()->payment)) {
            $this->_payment = Ecart_Payment::getMethod($this->getStorage()->payment);
        }
        return $this->_payment;
    }
    
    /**
     * Set payment method to session storage
     *
     * @param string $code Payment Method code
     */
    public function setPaymentMethod($code)
    {
        $this->getStorage()->payment = $code;
    }
    
    /**
     * Set shipping method to session storage
     *
     * @param string $code Shipping Method code
     */
    public function setShippingMethod($code)
    {
        $this->getStorage()->shipping = $code;
    }

    /**
     * Load data about country and zone
     * Used when address for checkout is not taken from database
     * 
     * @param array $address
     * @return array
     */
    protected function _prepareAddress(array $address)
    {
        if (!isset($address['country_id'])) {
            return false;
        }
        $address['country'] = Ecart::single('location/country')
                ->find($address['country_id'])
                ->current()
                ->toArray();
        if (isset($address['zone_id'])) {
            $address['zone'] = Ecart::single('location/zone')
                ->find($address['zone_id'])
                ->current()
                ->toArray();
        }
        return $address;
    }

    /**
     * Set delivery address
     *
     * @param mixed $address Address id or address array
     * @return bool
     */
    public function setDeliveryAddress($address)
    {
        if (!is_array($address)) { // existing address in db
            $address = Ecart::single('account/customer_address')->get($address);
        } else {
            $address = $this->_prepareAddress($address);
        }
        
        if (!$address) {
            return false;
        }
        
        $this->getStorage()->delivery = $address;
        return true;
    }

    /**
     * Set billing address
     *
     * @param mixed $address Address id or address array
     * @return bool
     */
    public function setBillingAddress($address)
    {
        if (!is_array($address)) { // existing address in db
            $address = Ecart::single('account/customer_address')->get($address);
        } else {
            $address = $this->_prepareAddress($address);
        }
        
        if (!$address) {
            return false;
        }
        
        $this->getStorage()->billing = $address; 
        return true;
    }
    
    /**
     * @return void
     */
    public function setOrderId($orderId)
    {
        $this->getStorage()->orderId = $orderId;
    }
    
    /**
     * @return int
     */
    public function getOrderId()
    {
        return $this->getStorage()->orderId;
    }
    
    /**
     * Validate that all data was initialized
     *
     * @return boolean
     */
    public function check()
    {
        $storage = $this->getStorage();
        $this->_errors = array();
        if (!Ecart::getCustomerId() && !$this->getStorage()->asGuest) {
            $this->_errors[] = array(
                'code' => self::ERROR_CUSTOMER,
                'text' => 'Unknown customer. Sorry but you must start checkout process again.'
            );
        }
        if (!isset($storage->delivery)) {
            $this->_errors[] = array(
                'code' => self::ERROR_DELIVERY,
                'text' => 'Empty shipping address'
            );
        }
        if (!isset($storage->shipping)) {
            $this->_errors[] = array(
                'code' => self::ERROR_SHIPPPING,
                'text' => 'Empty shipping method'
            );
        }
        if (!isset($storage->billing)) {
            $this->_errors[] = array(
                'code' => self::ERROR_BILLING,
                'text' => 'Empty billing address'
            );
        }
        if (!isset($storage->payment)) {
            $this->_errors[] = array(
                'code' => self::ERROR_PAYMENT,
                'text' => 'Empty payment method'
            );
        }
        
        if (count($this->_errors)) {
            return false;
        }
        
        return true;
    }
    
    /**
     * Return errors from check method
     *
     * @return array
     */
    public function getErrors()
    {
        return $this->_errors;
    }
}