<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Model_Total
{
    /**
     * Array of Ecart_Checkout_Model_Total_ methods
     *
     * @var array
     */
    private $_methods = array();
    
    /**
     * Collects array, content order_total information
     *
     * @var array
     */
    private $_collects = null;
    
    /**
     * Apply all total methods for current total
     * Fluent interface
     * @return Ecart_Checkout_Model_Total
     */
    public function collectTotals()
    {
        $this->_collects = array();
        foreach ($this->getMethods() as $method) {
            if (!$method->isEnabled()) {
                continue;
            }
            $method->collect($this);
        }
        return $this;
    }
    
    public function getCollects()
    {
        if (null === $this->_collects) {
            $this->collectTotals();
        }
        uasort($this->_collects, array($this, '_sortCollects'));
        return $this->_collects;
    }

    /**
     * Functions for sorting collects
     * @param $a
     * @param $b
     * @return int
     */
    private function _sortCollects($a, $b)
    {
        if ($a['sortOrder'] == $b['sortOrder']) {
            return 0;
        }
        return ($a['sortOrder'] > $b['sortOrder']) ? 1 : -1;
    }

    /**
     * Add new total collect
     *
     * @param array $collect
     */
    public function addCollect($collect)
    {
        $this->_collects[$collect['code']] = $collect;
    }
    
    /**
     * Retrieve final total sum for payment, including shipping price and taxes
     * 
     * @param string $code module code
     * @return float
     */
    public function getTotal($code = null)
    {
        if (!empty($code)) {
            if (null !== $this->_collects && isset($this->_collects[$code])) {
                $total = $this->_collects[$code]['total'];
            } else {
                $methodName = str_replace('Checkout_', '', $code);
                $method = $this->_getMethod($methodName);
                if ($method->isEnabled()) {
                    $method->collect($this);
                    $total = $this->_collects[$code]['total'];
                    $this->_collects = null;
                } else {
                    $total = 0;
                }
            }
        } else {
            if (null === $this->_collects) {
                $this->collectTotals();
            }
            $total = 0;
            foreach ($this->_collects as $collect) {
                $total += $collect['total'];
            }
        }
        return $total;
    }
    
    /**
     * Return list of order totals  
     * @return array
     */
    public function getMethodNames()
    {
        if ($methods = Ecart::cache()->load('order_total_methods')) {
            return $methods;
        }
        $dirPath = realpath(Ecart::config()->system->path . '/app/code/Ecart/Checkout/Model/Total');
        $methods = array();
        $skip = array('Abstract');
        $dp = opendir($dirPath);
        while ($fname = readdir($dp)) {
            if (!is_file("{$dirPath}/{$fname}")) {
                continue;
            }
            list($name, $ext) = explode('.', $fname, 2);
            if (in_array($name, $skip) || $ext != 'php') {
                continue;
            }
            $methods[] = $name;
        }
        closedir($dp);
        Ecart::cache()->save($methods, 'order_total_methods', array('modules'));
        return $methods;
    }
    
    /**
     * Retrieve Ecart_Checkout_Model_Total_ methods list
     * 
     * @return array
     */
    public function getMethods()
    {
        foreach ($this->getMethodNames() as $name) {
            if (!isset($this->_methods[$name])) {
                $this->_getMethod($name);
            }
        }
        return $this->_methods;
    }
    
    /**
     * Retrieve method by name. Method is saved to $_methods
     * 
     * @param string $name
     * @return Ecart_Checkout_Model_Total_Abstract
     */
    private function _getMethod($name)
    {
        if (!isset($this->_methods[$name])) {
            $className = 'Ecart_Checkout_Model_Total_' . ucfirst($name);
            $this->_methods[$name] = new $className();
        }
        return $this->_methods[$name];
    }
}