<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Community
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Community
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Community_Model_Review extends Ecart_Db_Table
{
    protected $_name = 'community_review';
    protected $_rowClass = 'Ecart_Community_Model_Review_Row';
    protected $_primary = 'id';
    protected $_dependentTables = array('Ecart_Community_Review_Mark');
    
    /**
     * Retrieve the list of cutomer reviews
     * including product information and prices
     * 
     * @param mixed $where
     * @param string $order
     * @param string $dir
     * @param int $limit
     * @param int $page
     * @param bool $all If true - review with any status will be fetched
     * @return array
     */
    public function getList(
                            $where = array(),
                            $order = 'cr.date_created',
                            $dir = 'DESC',
                            $limit = null,
                            $page = null,
                            $all = false)
    {
        $select = $this->getAdapter()->select()
            ->from(array('cr' => $this->_name), 
                array(new Zend_Db_Expr('SQL_CALC_FOUND_ROWS DISTINCT cr.id'), 'cr.*')
            )
            ->joinLeft(array('cp' => self::$_db_prefix . 'catalog_product'),
                'cr.product_id = cp.id',
                array('product_image_thumbnail' => 'image_thumbnail', 'product_price' => 'price'))
            ->joinLeft(array('cpd' => self::$_db_prefix . 'catalog_product_description'),
                'cp.id = cpd.product_id AND cpd.language_id = ' . Ecart_Locale::getLanguageId(),
                array('product_name' => 'name', 'product_image_seo_name' => 'image_seo_name'))
            ->joinLeft(array('ch' => self::$_db_prefix . 'catalog_hurl'),
                "cp.id = ch.key_id AND ch.key_type = 'p'",
                array('product_key_word' => 'key_word'))
            ->order($order . ' ' . $dir);
        
        if ($order == 'rating') {
            $select->joinLeft(array('crm' => self::$_db_prefix . 'community_review_mark'),
                'cr.id = crm.review_id',
                array('rating' => new Zend_Db_Expr('AVG(crm.mark)')))
            ->group('cr.id');
        }
        
        if (!$all) {
            $select->where("cp.is_active = 1 AND cr.status = 'approved'");
        }
        
        if (null !== $where) {
            if (!is_array($where)) {
                $where = array($where);
            }
            foreach ($where as $statement) {
                $select->where($statement);
            }
        }
        
        if (null !== $page && null !== $limit) {
            $select->limitPage($page, $limit);
        } elseif (null !== $limit) {
            $select->limit($limit);
        }
        
        $reviews = $this->getAdapter()->fetchAssoc($select);
        $count = $this->getAdapter()->fetchOne("SELECT FOUND_ROWS()");
        
        $products = array();
        foreach ($reviews as $review) {
            $products[$review['product_id']] = array(
                'price' => $review['product_price']
            );
        }
        Ecart::single('discount/discount')->fillDiscount($products);

        foreach ($reviews as &$review) {
            if (!isset($products[$review['product_id']]) ||
                !isset($products[$review['product_id']]['price_discount'])) {

                continue;
            }
            $review['product_price_discount'] =
                $products[$review['product_id']]['price_discount'];
        }

        $ratings = $this->loadRating(array_keys($reviews));
        foreach ($reviews as $key => &$review) {
            $review['ratings'] = $ratings[$key];
        }
        
        return array(
            'reviews' => array_values($reviews),
            'count' => $count
        );
    }
    
    /**
     * Retrieve the list of cutomer reviews
     * including product information and prices
     * 
     * @param mixed $where
     * @param string $order
     * @param string $dir
     * @param int $limit
     * @param int $page
     * @return array
     */
    public function getBackList(
                                $where = array(),
                                $order = 'cr.date_created',
                                $dir = 'DESC',
                                $limit = null,
                                $page = null)
    {
        $select = $this->getAdapter()->select()
            ->from(
                array('cr' => $this->_name),
                array(new Zend_Db_Expr('SQL_CALC_FOUND_ROWS DISTINCT cr.id'), 'cr.*')
            )
            ->joinLeft(
                array('cpd' => self::$_db_prefix . 'catalog_product_description'),
                'cr.product_id = cpd.product_id AND cpd.language_id = '
                    . Ecart_Locale::getLanguageId(),
                array('product_name' => 'name'))
            ->order($order . ' ' . $dir);
        
        if (null !== $where) {
            if (!is_array($where)) {
                $where = array($where);
            }
            foreach ($where as $statement) {
                $select->where($statement);
            }
        }
        
        if (null !== $page && null !== $limit) {
            $select->limitPage($page, $limit);
        } elseif (null !== $limit) {
            $select->limit($limit);
        }
        
        $reviews = $this->getAdapter()->fetchAssoc($select);
        $count = $this->getAdapter()->fetchOne("SELECT FOUND_ROWS()");

        return array(
            'reviews' => array_values($reviews),
            'count' => $count
        );
    }


    
    /**
     * Retrieve the list of cutomer reviews, excluding product information
     * 
     * @param mixed $where
     * @param string $order
     * @param string $dir
     * @param int $limit
     * @param int $page
     * @param bool $all If true - review with any status will be fetched
     * @return array
     */
    public function getTinyList($where = array(),
                                $order = 'cr.date_created',
                                $dir = 'DESC',
                                $limit = null,
                                $page = null,
                                $all = false)
    {
        $select = $this->getAdapter()->select()
            ->from(array('cr' => $this->_name),
                array(new Zend_Db_Expr('SQL_CALC_FOUND_ROWS DISTINCT cr.id'), 'cr.*'))
            ->order($order . ' ' . $dir);
        
        if ($order == 'rating') {
            $select->joinLeft(array('crm' => self::$_db_prefix . 'community_review_mark'),
                'cr.id = crm.review_id',
                array('rating' => new Zend_Db_Expr('AVG(crm.mark)')))
            ->group('cr.id');
        }
        
        if (!$all) {
            $select->where("cr.status = 'approved'");
        }
        
        if (null !== $where) {
            if (!is_array($where)) {
                $where = array($where);
            }
            foreach ($where as $statement) {
                $select->where($statement);
            }
        }
        
        if (null !== $page && null !== $limit) {
            $select->limitPage($page, $limit);
        } elseif (null !== $limit) {
            $select->limit($limit);
        }
        
        $reviews = $this->getAdapter()->fetchAssoc($select);
        $count = Ecart::db()->fetchOne("SELECT FOUND_ROWS()");
        $ratings = $this->loadRating(array_keys($reviews));
        
        foreach ($reviews as $key => &$review) {
            $review['ratings'] = $ratings[$key];
        }
        
        return array(
            'reviews' => array_values($reviews),
            'count' => $count
        );
    }
    
    /**
     * Load available rating marks to appropriate reviews
     * 
     * @param mixed $reviewIds
     * @return array
     */
    public function loadRating($reviewIds)
    {
        if (!is_array($reviewIds)) {
            $reviewIds = array($reviewIds);
        }
        
        if (!count($reviewIds)) {
            return array();
        }
        
        $select = $this->getAdapter()->select()
            ->from(
                array('crm' => self::$_db_prefix . 'community_review_mark'),
                array('review_id', 'mark')
            )
            ->joinRight(array('crr' => self::$_db_prefix . 'community_review_rating'),
                'crm.rating_id = crr.id',
                'name')
            ->joinLeft(array('crrt' => self::$_db_prefix . 'community_review_rating_title'),
                'crm.rating_id = crrt.rating_id AND crrt.language_id = ' . Ecart::getLanguageId(),
                'title')
            ->where('crm.review_id IN (?)', $reviewIds);
        
        $result = array();
        $ratings = array();
        
        foreach ($this->getAdapter()->fetchAll($select) as $rating) {
            $ratings[$rating['review_id']][$rating['name']] = $rating;
        }
        
        $available_ratings = Ecart::single('community/review_rating')->getList();
        
        foreach ($reviewIds as $id) {
            foreach ($available_ratings as $rating) {
                $result[$id][$rating['name']] = isset($ratings[$id][$rating['name']]) ?
                    $ratings[$id][$rating['name']] : array();
            }
        }
        
        return $result;
    }
    
    /**
     * Load average ratings by product
     * 
     * @param mixed $productIds
     * @param bool $mergeRatings
     * @return array (product_id => ratings)
     */
    public function getAverageProductRating($productIds, $mergeRatings = false)
    {
        if (!is_array($productIds)) {
            $productIds = array($productIds);
        }
        
        $select = $this->getAdapter()->select()
            ->from(
                array('crm' => self::$_db_prefix . 'community_review_mark'),
                array('average_mark' => new Zend_Db_Expr('AVG(crm.mark)'))
            )
            ->joinLeft(array('cr' => $this->_name),
                'cr.id = crm.review_id',
                array('product_id' => 'cr.product_id'))
            ->where('cr.product_id IN (?)', $productIds)
            ->group('cr.product_id');
        
        if (!$mergeRatings) {
            $select->joinLeft(array('crr' => self::$_db_prefix . 'community_review_rating'),
                    'crm.rating_id = crr.id',
                    'name')
                ->joinLeft(array('crrt' => self::$_db_prefix . 'community_review_rating_title'),
                    'crm.rating_id = crrt.rating_id AND crrt.language_id = ' . Ecart::getLanguageId(),
                    'title')
                ->group('crr.name')
                ->order('crrt.title DESC');
        }
        
        $result = array();
        foreach ($this->getAdapter()->fetchAll($select) as $averageMark) {
            $name = isset($averageMark['name']) ?
                $averageMark['name'] : 'rating';
            $result[$averageMark['product_id']][$name] = array(
                'title' => isset($averageMark['title']) ?
                    $averageMark['title'] : Ecart::translate('community')->__('Rating'),
                'mark' => round($averageMark['average_mark'], 1),
                'product_id' => $averageMark['product_id']
            );
        }
        
        return $result;
    }
    
    /**
     * Load average rating by customer
     * 
     * @param mixed $customerIds
     * @param bool $mergeRatings
     * @return array (customer_id => ratings)
     */
    public function getAverageCustomerRating($customerIds, $mergeRatings = false)
    {
        if (!is_array($customerIds)) {
            $customerIds = array($customerIds);
        }
        
        $select = $this->getAdapter()->select()
            ->from(
                array('crm' => self::$_db_prefix . 'community_review_mark'),
                array('average_mark' => new Zend_Db_Expr('AVG(crm.mark)'))
            )
            ->joinLeft(array('cr' => $this->_name),
                'cr.id = crm.review_id',
                array('customer_id' => 'cr.customer_id'))
            ->where('cr.customer_id IN (?)', $customerIds)
            ->group('cr.customer_id');
        
        if (!$mergeRatings) {
            $select->joinLeft(array('crr' => self::$_db_prefix . 'community_review_rating'),
                    'crm.rating_id = crr.id',
                    'name')
                ->joinLeft(array('crrt' => self::$_db_prefix . 'community_review_rating_title'),
                    'crm.rating_id = crrt.rating_id AND crrt.language_id = ' . Ecart::getLanguageId(),
                    'title')
                ->group('crr.name')
                ->order('crrt.title DESC');
        }
        
        $result = array();
        foreach ($this->getAdapter()->fetchAll($select) as $average_mark) {
            $name = isset($average_mark['name']) ? $average_mark['name'] : 'rating';
            $result[$average_mark['customer_id']][$name] = array(
                'title' => isset($average_mark['title']) ? 
                    $average_mark['title'] : Ecart::translate('community')->__('Rating'),
                'mark' => round($average_mark['average_mark'], 1),
                'customer_id' => $average_mark['customer_id']
            );
        }
        
        return $result;
    }
    
    /**
     * Add or update customers review
     * 
     * @param array $data
     *  (
     *    product_id => int,    required
     *    title => string,      required
     *    pros => string,       required
     *    cons => string,       required
     *    summary => string,    optional
     *    id => int             optional,
     *    status => (pending|approved|disapproved) optional
     *    ratings => array(     optional
     *      [1] => double,      rating_id => rating_mark pair
     *      ...
     *    )
     *  )
     * @return bool
     */
    public function save($data)
    {
        if (!isset($data['customer_id'])) {
            $data['customer_id'] = Ecart::getCustomerId() ?
                Ecart::getCustomerId() : new Zend_Db_Expr('NULL');
        } elseif (!is_numeric($data['customer_id']) 
                || !Ecart::single('account/customer')->find($data['customer_id'])->current()) {

            $data['customer_id'] = new Zend_Db_Expr('NULL');
        }
        if (!isset($data['status'])) {
            $data['status'] = $this->getDefaultStatus();
            if ($data['status'] != 'approved') {
                Ecart::message()->addSuccess(
                    Ecart::translate('community')->__(
                        'Review has been accepted for moderation'
                    )
                );
            }
        }
        
        if (!isset($data['id'])
            || empty($data['id'])
            || !$row = $this->find($data['id'])->current()) {
            
            $row = $this->createRow();
            $row->date_created = Ecart_Date::now()->toZendDbExpr();
        }
        unset($data['id']);
        $row->setFromArray($data);
        $row->save();
        
        Ecart::message()->addSuccess(
            Ecart::translate('community')->__(
                'Review was successfully saved'
        ));
        
        $data['ratings'] = array_filter($data['ratings']);
        
        if (!is_numeric($data['customer_id']) && count($data['ratings'])) {
            Ecart::message()->addNotice(
                Ecart::translate('community')->__(
                    'Guests do not have the permission to vote. Review was saved without ratings'
            ));
        } elseif (count($data['ratings']) && 
            Ecart::single('community/review_mark')->isCustomerVoted(
                $data['customer_id'], $data['product_id'], $row->id
            )) {

            Ecart::message()->addNotice(
                Ecart::translate('community')->__(
                    'You have already voted for this product. Review was saved without ratings'
            ));
        } elseif (count($data['ratings'])) {
            $row->saveMark($data['ratings']);
        }
        
        return true;
    }
    
    /**
     * Retrieve the default review status
     * @return string
     */
    public function getDefaultStatus()
    {
        if (Ecart::getCustomerId()) {
            return Ecart::config()->community->review->customer_status;
        } else {
            return Ecart::config()->community->review->guest_status;
        }
    }
    
    /**
     * Returns can or not current user write a review
     * @return bool
     */
    public function canAdd()
    {
        if (!Ecart::config()->community->review->guest_permission 
            && !Ecart::getCustomerId()) {
            
            return false;
        }
        return true;
    }
    
    /**
     * Remove reviews by ids
     * 
     * @param mixed $ids
     * @return void
     */
    public function remove($ids)
    {
        if (!is_array($ids)) {
            $ids = array($ids);
        }
        $where = $this->getAdapter()->quoteInto('id IN (?)', $ids);
        $this->delete($where);
        Ecart::message()->addSuccess(
            Ecart::translate('community')->__(
                "%d review(s) was deleted successfully", count($ids)
        ));
    }
    
    /**
     * Retrieve reviews count for specified product(s)
     * 
     * @param mixed $ids
     * @return array (productId => count,...)
     */
    public function getCountByProductId($ids)
    {
        if (!is_array($ids)) {
            $ids = array($ids);
        }
        
        $select = $this->select()->from(array('cr' => $this->_name), 
            array('id' => 'product_id', 'count' => new Zend_Db_Expr('COUNT("id")')))
            ->where('product_id IN (?)', $ids)
            ->group('product_id');
        
        $counts = $this->getAdapter()->fetchAssoc($select);
        $result = array();
        foreach ($ids as $productId) {
            $result[$productId] = isset($counts[$productId]) ?
                $counts[$productId]['count'] : 0;
        }
        return $result;
    }
}