<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Core
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Core_Model_Template_Box extends Ecart_Db_Table
{
    protected $_name = 'core_template_box';
    
    /**
     * Retrieve the list of all boxes, including active cms boxes
     * 
     * @param bool $includeCms
     * @return array
     */
    public function getList($includeCms = true)
    {
        $modules = Ecart::single('core/module')->getList();
        
        if (!($boxes = Ecart::cache()->load('boxes_list'))) {
            $boxes = array();
            foreach ($modules as $moduleCode => $values) {
                list($category, $module) = explode('_', $moduleCode, 2);
                $path = Ecart::config()->system->path 
                      . '/app/code/' . $category . '/' . $module . '/Box';

                if (!is_readable($path)) {
                    continue;
                }
                $dir = opendir($path);
                while (($file = readdir($dir))) {
                    if (!is_readable($path . '/' . $file) 
                        || !is_file($path . '/' . $file)) {

                        continue;
                    }
                    $box = substr($file, 0, strpos($file, '.'));

                    if (in_array($box, array('Abstract', 'Block'))) {
                        continue;
                    }
                    $boxes[] = ucwords($category . '_' . $module . '_' . $box);
                    
                }
            }
            Ecart::cache()->save($boxes, 'boxes_list', array('modules'));
        }
        
        /* this part is not needed to be cached */
        if ($includeCms && in_array('Ecart_Cms', array_keys($modules))) {
            $cmsBoxs = Ecart::single('admin/cms_block')
                ->getBlocks('is_active = 1');
            foreach ($cmsBoxs as $box) {
                $boxes[] = 'Ecart_Cms_Block_' . $box['name'];
            }
        }
        sort($boxes);
        
        return $boxes;
    }
    
    /**
     * Retrieve the information about boxes within provided pages and template
     * 
     *  @param int $templateId
     *  @param  array $pagesIds
     *  @return array Boxes and nforamtion about them
     */
    public function getCustomInfo($templateId, $pagesIds) 
    {
        return $this->getAdapter()->fetchAll(
            "SELECT tbox.id, tbox.class, tbox.block, tbox.config,"
               . "tbtp.box_show, tbtp.sort_order, tbtp.block as other_block, tbtp.template, tbtp.tab_container, tbtp.page_id "
          . "FROM {$this->_name} tbox "
          . "INNER JOIN " . self::$_db_prefix . "core_template_box_page tbtp ON tbtp.box_id = tbox.id "
          . "WHERE tbox.template_id = ? AND "
              . "tbox.box_status = 1 AND "
              . $this->getAdapter()->quoteInto(" tbtp.page_id IN(?)", $pagesIds)
          . "ORDER BY tbox.sort_order",
            $templateId
        );
    }
    
    /**
     * Save or insert box data
     * 
     * @param int $templateId
     * @param array $data
     * @param string(normal|batch) mode
     * @return bool
     */
    public function save($templateId, $data, $mode = 'normal')
    {
        if (!sizeof($data)) {
            Ecart::message()->addError(
                Ecart::translate('core')->__(
                    'No data to save'
            ));
            return false;
        }
        
        foreach ($data as $id => $item) {
            if (!isset($item['id']) 
                || !$row = $this->find($item['id'])->current()) {

                $row = $this->createRow();
                $row->template_id = $templateId;
            } else {
                unset($item['id']);
            }
            $row->setFromArray($item);
            $row->save();
            
            /* insert assignments to pages */
            $boxPages = Ecart::single('core/template_box_page');
            if ($mode != 'batch') {
                $boxPages->delete('box_id = ' . intval($row->id));
                foreach ($item['box'] as $pageId => $box) {
                    if (isset($item['show'][$pageId]) 
                        && $item['show'][$pageId]) {

                        $show = 1;
                    } elseif (isset($item['hide'][$pageId]) 
                        && $item['hide'][$pageId]) {

                        $show = 0;
                    } else {
                        continue;
                    }
                    $newRow = array(
                        'box_id' => $row->id,
                        'box_show' => $show,
                        'page_id' => $pageId,
                        'sort_order' => $row->sort_order,
                        'tab_container' => $box['tab_container'],
                        'template' => $box['template'],
                        'block' => $box['block']
                    );
                    $boxPages->insert($newRow);
                }
            } else {
                // get rows that remains visible
                $pageIds = explode(',', trim($item['show'], ', '));
                $boxIds = array_fill(0, count($pageIds), $row->id);
                $rowsToUpdate = $boxPages->find($boxIds, $pageIds);
                
                // delete all others visible
                $boxPages->delete(array(
                    'box_id = ' . intval($row->id),
                    'box_show = 1',
                    $this->_db->quoteInto('page_id NOT IN (?)', $pageIds)
                ));
                
                // update existing
                foreach ($rowsToUpdate as $rowToUpdate) {
                    unset(
                        $pageIds[array_search($rowToUpdate->page_id, $pageIds)]
                    );
                    $rowToUpdate->setFromArray(array(
                        'sort_order' => $row->sort_order
                    ));
                    $rowToUpdate->save();
                }
                
                // insert new
                foreach ($pageIds as $pageId) {
                    $boxPages->insert(array(
                        'box_id'   => $row->id,
                        'sort_order' => $row->sort_order,
                        'page_id'  => $pageId,
                        'box_show' => 1
                    ));
                }
            }
        }
        
        Ecart::message()->addSuccess(
            Ecart::translate('core')->__(
                'Box was saved successfully'
        ));
        return true;
    }
    
    /**
     * Insert box
     * @param string class 'Catalog_Navigation'
     * @param string block 'left' or  'footer' , etc... 
     * @param mixed $pages 
     *  'module/controler/action',
     *  array(
     *      'module/controler/action', 
     *      'module2/controler/action'
     *  )
     * @param int $templateId If template is not provided, current templateId will be used
     * @param int boxStatus 1 or 0 default 1
     * @param int sort_order 100 (default '100')
     * @param array $extraOptions
     *  'module/controler/action' => array(
     *      'box_show' => 0|1,
     *      'template' => 'template',
     *      'tab_container' => 'container',
     *      'block' => 'block'
     *      'sort_order' => sort_order
     *  )
     * @return Ecart_Core_Model_Template_Box Provides fluent interface
     */
    public function add(
        $class,
        $block = 'content',
        $pages = '*/*/*',
        $sortOrder = 100,
        $config = '',
        $templateId = null,
        $boxStatus = 1,
        $extraOptions = array())
    {
        if (!is_array($pages)) {
            $pages = array($pages);
        }
        if (null === $templateId) {
            $templateId = isset(Ecart::config()->design->main->frontTemplateId) ? 
                Ecart::config()->design->main->frontTemplateId : 1;
        }
        $data = array(
            'block' => $block,
            'class' => $class,
            'sort_order' => $sortOrder,
            'box_status' => $boxStatus,
            'template_id' => $templateId,
            'config' => $config
        );
        $boxId = $this->insert($data);
        
        foreach ($pages as $page) {
            $pageId = Ecart::single('core/page')->getPageIdByRequest($page);
            if (!$pageId) {
                $request = explode('/', $page);
                $pageId = Ecart::single('core/page')->insert(array(
                    'module_name' => $request[0],
                    'controller_name' => $request[1],
                    'action_name' => $request[2]
                ));
            }
            Ecart::single('core/template_box_page')->insert(array(
                'box_id' => $boxId,
                'page_id' => $pageId,
                'box_show' => $data['box_status'],
                'template' => new Zend_Db_Expr('NULL'),
                'block' => new Zend_Db_Expr('NULL'),
                'sort_order' => $data['sort_order']
            ));
        }
        
        foreach ($extraOptions as $page => $values) {
            $pageId = Ecart::single('core/page')->getPageIdByRequest($page);
            
            if (!$pageId) {
                $request = explode('/', $page);
                $pageId = Ecart::single('core/page')->insert(array(
                    'module_name' => $request[0],
                    'controller_name' => $request[1],
                    'action_name' => $request[2]
                ));
                // there are no boxes assigned to page, if it was just created, 
                $row = Ecart::single('core/template_box_page')->createRow();
            } elseif (!$row = Ecart::single('core/template_box_page')
                ->find($boxId, $pageId)->current()) {

                $row = Ecart::single('core/template_box_page')->createRow();
            }
            $row->setFromArray($values);
            $row->box_id = $boxId;
            $row->page_id = $pageId;
            $row->save();
        }
        
        return $this;
    }
    
    /**
     * Remove box 
     * @param string className (ModuleName_BoxName | ModuleName)
     * @return Ecart_Core_Model_Template_Box provides fluent interface
     */
    public function remove($className)
    {   
        list($category, $moduleName, $boxName) = explode('_', $className);
        
        if (empty($boxName)) {
            $boxName = '%';
        }
        if (empty($moduleName)) {
            $moduleName = '%';
        }
        
        $this->delete("class LIKE '{$category}_{$moduleName}_{$boxName}'");
        
        return $this;
    }
}