<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Sales
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Sales
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Sales_Model_Order extends Ecart_Db_Table
{
    protected $_name = 'sales_order';
    protected $_rowClass = 'Ecart_Sales_Model_Order_Row';
    
    /**
     * Create order, and clears shopping cart after that
     *
     * @return Ecart_Sales_Model_Order_Row
     * @throws Ecart_Exception
     */
    public function createFromCheckout()
    {
        
        $checkout = Ecart::single('checkout/checkout');
        /**
         * @var Ecart_Checkout_Model_Checkout  $checkout
         */
         
        if (!$checkout->getCart()->validateContent()) {
            throw new Ecart_Exception();
        }
        
        $orderRow = $this->createRow();
        /* Fill order row */

        $storage = $checkout->getStorage();
        if ($customerId = Ecart::getCustomerId()) {
            $customer = Ecart::single('account/customer')
                ->find($customerId)->current();
            $orderRow->customer_id = $customer->id;
            $orderRow->customer_email = $customer->email;
        } elseif($storage->asGuest && !$storage->registerGuest) {
            $billing = $storage->billing;
            $orderRow->customer_email = $billing['email'];
        }
       
        
        $orderRow->site_id = Ecart::getSiteId();
        
        $orderRow->payment_method = $checkout->payment()->getTitle();
        $orderRow->payment_method_code = $checkout->payment()->getCode();
        
        $orderRow->shipping_method = $checkout->shipping()->getTitle();
        $orderRow->shipping_method_code = $checkout->shipping()->getCode();
        
        $orderRow->date_purchased_on = Ecart_Date::now()->toZendDbExpr();
        $orderRow->currency = Ecart::single('locale/currency')
            ->getCurrentCurrency();
        $orderRow->currency_rate = Ecart::single('locale/currency')->getData(
            $orderRow->currency, 'rate'
        );
        $orderRow->order_total = $checkout->getTotal()->getTotal();
        $orderRow->txn_id = 0; //@todo
        $orderRow->order_status_id = 0;
        $orderRow->ip_address = $_SERVER['REMOTE_ADDR'];
        
        /* build delivery & billing arrays */
        $delivery = $checkout->getStorage()->delivery;
        $delivery['country'] = isset($delivery['country']['name']) ? 
            $delivery['country']['name'] : '';

        if (isset($delivery['zone']['name'])) {
            $delivery['state'] = $delivery['zone']['name'];
        }
        
        $billing = $checkout->getStorage()->billing;
        
        $billing['country'] = isset($billing['country']['name'])
            ? $billing['country']['name'] : '';
        if (isset($billing['zone']['name'])) {
            $billing['state'] = $billing['zone']['name'];
        }
        unset($billing['id'], $delivery['id']);
        
        foreach ($delivery  as $key => $value) {
            $delivery['delivery_' . $key] = $value;
        }
        foreach ($billing  as $key => $value) {
            $billing['billing_' . $key] = $value;
        }
        
        $orderRow->setFromArray($delivery);
        $orderRow->setFromArray($billing);
        
        /* Save order */
        $orderRow->save();
        
        /* Add products to order and change quantity */
        foreach ($checkout->getCart()->getProducts() as $product) {
            $this->insertProduct($product, $orderRow->id);
        }
        
        /* Add total info */
        $total = $checkout->getTotal();
        $orderTotal = Ecart::single('sales/order_total');
        foreach ($total->getCollects() as $collect) {
            $orderTotal->insert(array(
                'order_id' => $orderRow->id,
                'title'    => $collect['title'],
                'value'    => $collect['total']
            ));
        }
        
        // SET ORDER STATUS
        // set order status 'pending' or 'processing'
        if ($checkout->payment()->config('orderStatusId')) {
            $orderStatusId = $checkout->payment()->config('orderStatusId');
        } else {
            $orderStatusId = Ecart::config()->orders->main->defaultStatusId;
        }
        
        //@todo make chain of statuses, and apply it
        if (!in_array($orderStatusId, array(1, 'pending', 2 , 'processing'))) {
            $orderStatusId = 'pending';
        } elseif (in_array($orderStatusId, array(2, 'processing'))) {
            $orderRow->setStatus('pending');
        }

        $orderRow->setStatus($orderStatusId);
        /* Clear cart after checkout */
        $checkout->getCart()->clear();
        return $orderRow;
    }

    /**
     *
     * @param string $where [optional]
     * @return float
     */
    public function getTotal($where = null)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('o' => $this->_name), array("SUM(order_total)"));
            if (is_string($where) && $where) {
                $select->where($where);
            } elseif (is_array($where)) {
                foreach ($where as $condition) {
                    if ($condition)
                        $select->where($condition);
                }
            } elseif ($where instanceof Zend_Db_Select) {
        }
        $return = $this->getAdapter()->fetchOne($select->__toString());
        return $return ? $return : 0;
    }
    
    /**
     * Return count of records in orders table
     *
     * @param array $params [optional]
     * @param Zend_Db_Select $select
     * @return integer
     */
    public function countByFilter(
        $params = array(), Zend_Db_Select $select = null)
    {
        $select = $this->getAdapter()->select();
        $select->from(
            array('o' => $this->_name),
            new Zend_Db_Expr('COUNT(o.id)')
        );
        if ($params['filter']) {
            $select->joinLeft(
                array('c' => self::$_db_prefix . 'account_customer'),
                'o.customer_id = c.id',
                array()
            );
        }
        return parent::countByFilter($params, $select);
    }
    
    /**
     * Add product to order
     * 
     * @param array $product
     * @param int $orderId
     * @return int orderProductId
     */
    public function insertProduct($product, $orderId)
    {
        $stock = Ecart::single('catalog/product_stock')
            ->find($product['product_id'])->current();
        $backorder = $stock->backorder;
        $productVariationId = isset($product['product_variation_id']) ?
            $product['product_variation_id'] : null;

        $stockQuantity = $stock->getQuantity($productVariationId);
        if ($stockQuantity - $product['quantity'] > $stock->min_qty) {
            $backorder = 0;   
        }
        
        //@TODO  з ЦИМ  тахсом щось тра буде зробити | Що саме?
        $countryName = Ecart::single('sales/order')->getDeliveryCountry($orderId);
        $zoneName = Ecart::single('sales/order')->getDeliveryState($orderId);
        $countryId = Ecart::single('location/country')->getIdByName($countryName);
        $zoneId = 0;
        if (!empty($zoneName)) {
            $zoneId = Ecart::single('location/zone')->getIdByName($zoneName);
        }

        $geozoneIds = Ecart::single('location/geozone')->getIds($countryId, $zoneId);
        $customerId = Ecart::single('sales/order')->getCustomerId($orderId);//Ecart::getCustomerId();
        $customerGroupId = Ecart::single('account/customer')->getGroupId($customerId);
        $taxClassId = Ecart::single('catalog/product')->getTaxClassId($product['product_id']);

        $productTax = Ecart::single('tax/rate')->calculateByPrice(
            $product['price_final'], 
            $taxClassId,
            $geozoneIds,
            $customerGroupId
        );
        
        $orderProductId = Ecart::single('sales/order_product')->insert(array(
            'order_id'          => $orderId,
            'product_id'        => $product['product_id'],
            'product_variation_id' => $productVariationId,
            'product_sku'       => $product['sku'],
            'product_name'      => $product['name'],
            'product_price'     => $product['price'],
            'product_tax'       => $productTax,
            'final_price'       => $product['price_final'],
            'product_quantity'  => $product['quantity'],
            'backorder'         => $backorder 
        ));
        
       // $orderProductId = $this->getAdapter()->lastInsertId();
        foreach ($product['attributes'] as $attr) {
            Ecart::single('sales/order_product_attribute')->insert(array(
                'order_product_id' => $orderProductId,
                'product_option' => $attr['name'],
                'product_option_value' => $attr['value'],
            ));
        }
        $productRow = Ecart::single('catalog/product')
            ->find($product['product_id'])->current();
        $productRow->ordered += 1;//$product['quantity'];
        $productRow->save();
        return $orderProductId;
    }

    /**
     *
     * @param int       $orderId
     * @param int       $statusId
     * @param string    $comments
     * @param bool      $notify
     * @return mixed
     */
    public function insertStatusHistory(
        $orderId, $statusId, $comments, $notify = false)
    {
        return Ecart::single('sales/order_status_history')->insert(array(
            'order_id' => $orderId,
            'order_status_id' => $statusId,
            'created_on' => Ecart_Date::now()->toZendDbExpr(),
            'notified' => $notify ? 1 : 0,
            'comments' => $comments
        ));
    }

    /**
     *
     * @param int $orderId
     * @return array
     */
    public function getProducts($orderId)
    {
        $products = $this->getAdapter()->fetchAssoc(
            "SELECT * FROM " . self::$_db_prefix
                . "sales_order_product WHERE order_id = ?", $orderId
        );
        
        /* select attributes */
        $attributes = $this->getAdapter()->fetchAll(
          "SELECT opa.* FROM "
                . self::$_db_prefix . "sales_order_product_attribute opa "
          . "INNER JOIN " . self::$_db_prefix
                . "sales_order_product op ON op.id = opa.order_product_id "
          . "WHERE op.order_id = ?", $orderId
        );
        foreach ($attributes as $attr) {
            $products[$attr['order_product_id']]['attrs'][] = $attr;
        }
        
        return $products;
    }

    /**
     *
     * @param   int     $orderId
     * @return  float
     */
    public function getShipping($orderId)
    {
        return Ecart::single('sales/order_total')->getShipping($orderId);
    }

    /**
     *
     * @param   int     $orderId
     * @return  float
     */
    public function getTax($orderId)
    {
        return Ecart::single('sales/order_total')->getTax($orderId);
    }

    /**
     *
     * @param   int     $orderId
     * @return  float
     */
    public function getShippingTax($orderId)
    {
        return Ecart::single('sales/order_total')->getShippingTax($orderId);
    }

    /**
     *
     * @param   int     $orderId
     * @return  float
     */
    public function getSubtotal($orderId)
    {
        return Ecart::single('sales/order_total')->getSubtotal($orderId);
    }

    //@todo remove this fuction
    /**
     *
     * @param int $orderId
     * @return array
     */
    public function getAddresses($orderId)
    {
        $order = $this->find($orderId)->current()->toArray();
        return array(
            'billing' => array(
                'firstname'      => $order['billing_firstname'],
                'lastname'       => $order['billing_lastname'],
                'street_address' => $order['billing_street_address'],
                'city'           => $order['billing_city'],
                'postcode'       => $order['billing_postcode'],
                'zone'           => array('name' => $order['billing_state']),
                'country'        => array('name' => $order['billing_country'])
            ),
            'delivery' => array(
                'firstname'      => $order['delivery_firstname'],
                'lastname'       => $order['delivery_lastname'],
                'street_address' => $order['delivery_street_address'],
                'city'           => $order['delivery_city'],
                'postcode'       => $order['delivery_postcode'],
                'zone'           => array('name' => $order['delivery_state']),
                'country'        => array('name' => $order['delivery_country'])
            )
        );
    }

    /**
     *
     * @param array $params
     * @return array
     */
    public function getList($params = array())
    {
        $select = $this->getAdapter()->select();
        $select->from(array('o' => $this->_name));
        
        if (!empty($params['sort'])) {
            $select->order($params['sort'] . ' ' . $params['dir']);
        }
        if (!empty($params['limit'])) {
            $select->limit($params['limit'], $params['start']);
        }
            
        if (isset($params['filter']) && is_array($params['filter'])) {
            foreach ($params['filter'] as $filter) {
                switch ($filter['data']['type']) {
                    case 'numeric': 
                    case 'date':
                        $condition = $filter['data']['comparison'] == 'eq' ?
                            '=' : ($filter['data']['comparison'] == 'noteq' ?
                                '<>' : ($filter['data']['comparison'] == 'lt' ?
                                    '<' : '>'));
                        $select->where(
                            "o.{$filter['field']} $condition ?",
                            $filter['data']['value']
                        );
                        break;
                    case 'list':
                        $select->where($this->getAdapter()->quoteInto(
                            "o.$filter[field] IN (?)",
                            explode(',', $filter['data']['value'])
                        ));
                        break;
                    default:
                        $select->where(
                            "o.$filter[field] LIKE ?",
                            $filter['data']['value'] . "%"
                        );
                        break;
                }
            }
        }
        //Zend_Debug::dump($select->__toString());
        return $select->query()->fetchAll();
    }

    /**
     *
     * @param int $period [optional]
     * @param string $where [optional]
     * @param bool $distinctCustomer [optional]
     * @return array
     */
    public function getCountList(
        $period = 10, $where = null, $distinctCustomer = false)
    {
        /**
         *  7 -month
         *  10 - days
         *  13 - hour
         *  16 - minute
         */
        $select = $this->getAdapter()->select();
        if ($distinctCustomer) {
            $select->from(
                array('o' => $this->_name),
                array("LEFT(date_purchased_on, {$period}) AS pdate",
                    'COUNT(DISTINCT customer_email) as hit'
                )
            );
        } else {
            $select->from(
                array('o' => $this->_name),
                array("LEFT(date_purchased_on, {$period}) AS pdate",
                    'COUNT(*) as hit'

                )
            );
        }
        $select->group('pdate')
               ->order('pdate ASC');
        if (is_string($where) && $where) {
            $select->where($where);
        } elseif (is_array($where)) {
            foreach ($where as $condition) {
                if ($condition)
                    $select->where($condition);
            }
        }
        return $this->getAdapter()->fetchPairs($select->__toString());
    }

    /**
     *
     * @param int $period
     * @param string $where
     * @return array
     */
    public function getAmountsList($period = 10, $where = null)
    {
        $select = $this->getAdapter()->select();
        $select->from(
                array('o' => $this->_name),
                array("LEFT(date_purchased_on, {$period}) AS pdate")
            )
            ->joinLeft(
                array('op' => self::$_db_prefix . 'sales_order_product'),
                'op.order_id = o.id', array('SUM(op.product_quantity)')
            )
            ->group('pdate')
            ->order('pdate ASC');
        if (is_string($where) && $where) {
            $select->where($where);
        } elseif (is_array($where)) {
            foreach ($where as $condition) {
                $select->where($condition);
            }
        }
        return $this->getAdapter()->fetchPairs($select->__toString());
    }

    /**
     *
     * @return int
     */
    public function getLastOrderId()
    {
        return $this->getAdapter()->fetchOne(
            "SELECT id FROM {$this->_name} ORDER BY id DESC LIMIT 1"
        );
    }

    /**
     *
     * @param int $customerId
     * @param int $siteId
     * @return int
     */
    public function getOrderByCustomer($customerId, $siteId = null)
    {
        if (null === $siteId) {
            $siteId = Ecart::getSiteId();
        }
        $orderStatuses = array();

        $orderStatusRowset = Ecart::single('sales/order_status_text')->getList();
        foreach ($orderStatusRowset as $orderStatus) {
            $orderStatuses[$orderStatus['status_id']] = $orderStatus;
        }
            
        $orders = $this->fetchAll(array(
            $this->getAdapter()->quoteInto('customer_id = ?', $customerId),
            $this->getAdapter()->quoteInto('site_id = ?', $siteId)
        ), 'date_purchased_on DESC')->toArray();
        
        foreach ($orders as &$order) {
            
            $order['order_total'] = Ecart::single('locale/currency')->from(
                $order['order_total'] * $order['currency_rate'],
                $order['currency']
            );

            $order['order_status_name'] = isset(
                $orderStatuses[$order['order_status_id']]) ?
                    $orderStatuses[$order['order_status_id']]['status_name'] :
                    Ecart::translate('sales')->__('Undefined');
        }
        return $orders;
    }

    /**
     *
     * @param int $siteId [optional]
     * @return string
     */
    public function getFirstDate($siteId = null)
    {
        $select = "SELECT MIN(date_purchased_on) FROM `{$this->_name}`";
        if (null != $siteId) {
            $select .= " WHERE (site_id = '{$siteId}')";
        }
        return $this->getAdapter()->fetchOne($select);

    }

    /**
     *
     * @param int $siteId [optional]
     * @return string
     */
    public function getLastDate($siteId = null)
    {
        $select = "SELECT MAX(date_purchased_on) FROM `{$this->_name}`";
        if (null != $siteId) {
            $select .= " WHERE (site_id = '{$siteId}')";
        }
        return $this->getAdapter()->fetchOne($select);
    }
    
}