<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Sales
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Sales
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Sales_Model_Order_Row extends Ecart_Db_Table_Row
{
    /**
     * Set order status & insert information to order status history
     *
     * @param integer|string $statusId
     * @param string $comments
     * @param bool $notifyCustomer
     * @return bool
     */
    public function setStatus($statusId, $comments = '', $notifyCustomer = false)
    {
        if (!$this->id) {
            Ecart::message()->addError(
                Ecart::translate('sales')->__(
                    'Order not found'
            ));
            return false;
        }
        
        if (is_string($statusId) 
            && in_array($statusId, array_values(Ecart_Collect_OrderStatus::collect()))) {
                
            $status = $statusId;
            $statusId = Ecart::single('sales/order_status')->getIdByName($status);
        } else {
            $status = Ecart_Collect_OrderStatus::getName($statusId);
        }
        
        /* Relation exist or not */
        $childrens = Ecart::single('sales/order_status_relation')
            ->getChildrens($this->order_status_id);
        if (!in_array($statusId, $childrens)) {
            Ecart::message()->addError(
                Ecart::translate('sales')->__(
                    'Relation not exist %s <=> %s',
                    $this->order_status_id,
                    $statusId
            ));
            return false;
        }
        
        $retMethod = Ecart::single('sales/order_status_run')->$status($this);
        
        if (!$retMethod) {
            $status = 'failed';
            Ecart::single('sales/order_status_run')->failed($this);
            $statusId = Ecart::single('sales/order_status')->getIdByName('failed');
            $notifyCustomer = false;
            $messages = array();
            foreach (Ecart::message()->get() as $messageGroup) {
                $comments .= ' ' . implode('.', $messageGroup);
            }
        }
        
        $this->order_status_id = $statusId;
        $this->save();
        
        /* save order status history */
        Ecart::single('sales/order_status_history')->insert(array(
            'order_id' => $this->id,
            'order_status_id' => $statusId,
            'created_on' => Ecart_Date::now()->toZendDbExpr(),
            'notified' => (int) $notifyCustomer,
            'comments' => $comments
        ));
        
        if ($notifyCustomer) {
            $this->_notify($comments);
        }
        $message = Ecart::translate('sales')->__(
            "Order status was changed to %s", $status
        );
        if ($status == 'failed' && Zend_Registry::get('app') == 'front') {
            Ecart::message()->addError($message);
            if (!$retMethod) {
                Ecart::message()->addError($comments);
            }
        } else {
            Ecart::message()->addSuccess($message);
        }
        return $retMethod;
    }
    
    /**
     * Add comment to order(not change order status)
     * Fluent interface
     * @param $comment string 
     * @param $notify bool 
     * @return this
     */
    public function addComment($comment, $notify = false) 
    {
        Ecart::single('sales/order_status_history')->insert(array(
            'order_id' => $this->id,
            'order_status_id' => $this->order_status_id,
            'created_on' => Ecart_Date::now()->toZendDbExpr(),
            'notified' => (int) $notify,
            'comments' => $comment
        ));
        if ($notify) {
            $this->_notify($comment);
        }
        return $this;
    }
    
    /**
     * Notify customer
     * @return unknown_type
     */
    protected function _notify($comments)
    {
        $order = Ecart::single('sales/order')->find($this->id)->current();
        $customerId = Ecart::single('sales/order')->getCustomerId($this->id);
        $customer = Ecart::single('account/customer')->find($customerId)->current();
        $status = Ecart::single('sales/order_status_text')->find(
            $this->order_status_id, Ecart::getLanguageId()
        )->current()->toArray();
        $status = $status['status_name'];
        $to = $customer->toArray();
        $to = $to['email'];
        
        $mail = new Ecart_Mail();
        
        $mail->setConfig(array(
            'event'   => 'change_order_status-customer',
            'subject' => 'Your order change status notify',
            'data'    => array(
                'firstname' => $customer->firstname,
                'lastname'  => $customer->lastname,
                'order'     => $order->toArray(),
                'comments'   => $comments,
                'status'    => $status
            ),
            'to'      =>  $to,
            'type'    => 'html'   
        ));
        $mail->send();
    }

    /**
     *
     * @return array
     */
    public function getStatusHistory()
    {
        return $this->getAdapter()->fetchAll(
            "SELECT osh.*, os.status_name "
          . "FROM " . self::$_db_prefix . "sales_order_status_history osh " 
          . "INNER JOIN " . self::$_db_prefix . "sales_order_status_text os ON os.status_id = osh.order_status_id "
          . "WHERE osh.order_id = ? AND os.language_id = ? " 
          . "ORDER BY osh.id desc", array($this->id, Ecart::getLanguageId())
        );
    }
    
    public function getProducts()
    {
        return $this->getTable()->getProducts($this->id);
    }

    /**
     * @return array
     */
    public function getTotals()
    {
        $totals = array();
        $totalRowset = Ecart::single("sales/order_total")
            ->fetchAll("order_id = " . intval($this->id))->toArray();
        foreach ($totalRowset as $total)
        {
            $totals[$total['title']] = $total;   
        }
        return $totals; 
    }
    
    public function getAddresses()
    {
        return $this->getTable()->getAddresses($this->id);
    }
    
    public function getShipping()
    {
        return $this->getTable()->getShipping($this->id);
    }
    
    public function getTax()
    {
        return $this->getTable()->getTax($this->id);
    }
    
    public function getShippingTax()
    {
        return $this->getTable()->getShippingTax($this->id);
    }

    public function getTaxAmount()
    {
        return $this->getTax() + $this->getShippingTax();
    }
    
    public function getSubtotal()
    {
        return $this->getTable()->getSubtotal($this->id);
    }

    /**
     *
     * @return int
     */
    public function getStatus()
    {
        return $this->order_status_id;
    }

    /**
     *
     * @return float
     */
    public function getAmount()
    {
        return $this->order_total;
    }

    /**
     *
     * @return Ecart_Object
     */
    public function getBilling()
    {
        $billing = new Ecart_Object();
        $billing->setFirstname($this->billing_firstname)
            ->setLastname($this->billing_lastname)
            ->setCompany($this->billing_company)
            ->setStreet($this->billing_street_address)
            ->setStreet2($this->billing_suburb)
            ->setCity($this->billing_city)
            ->setState($this->billing_state)
            ->setZip($this->billing_postcode)
            ->setCountry($this->billing_country)
            ->setCountryCode(Ecart::single('location/country')->getIsoCode2ByName(
                $this->billing_country
            ))
            ->setPhone($this->billing_phone)
            ->setFax($this->billing_fax)
            ->setAddressFormat($this->billing_address_format_id)
            ->setCustomerId($this->customer_id)
            ->setTaxId(/*@todo*/);
            
        if (null !== $this->billing_state) {
            $billing->setStateCode(Ecart::single('location/zone')->getCodeByName(
                $this->billing_state
            ));
        }
        
        return $billing;
    }

    /**
     *
     * @return Ecart_Object
     */
    public function getDelivery()
    {
        $delivery = new Ecart_Object();
        $delivery->setFirstname($this->delivery_firstname)
            ->setLastname($this->delivery_lastname)
            ->setCompany($this->delivery_company)
            ->setStreet($this->delivery_street_address)
            ->setStreet2($this->delivery_suburb)
            ->setCity($this->delivery_city)
            ->setState($this->delivery_state)
            ->setZip($this->delivery_postcode)
            ->setCountry($this->delivery_country)
            ->setCountryCode(Ecart::single('location/country')->getIsoCode2ByName(
                $this->delivery_country
            ))
            ->setPhone($this->delivery_phone)
            ->setFax($this->delivery_fax)
            ->setAddressFormat($this->delivery_address_format_id)
            ->setCustomerId($this->customer_id)
            ->setTaxId(/*@todo*/);
            
        if (null !== $this->delivery_state) {
            $delivery->setStateCode(Ecart::single('location/zone')->getCodeByName(
                $this->delivery_state
            ));
        }
        
        return $delivery;
    }

    /**
     *
     * @return int
     */
    public function getIp()
    {
        return $this->ip_address;
    }

    /**
     *
     * @return string
     */
    public function getCustomerEmail()
    {
        return $this->customer_email;
    }
}