<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @category    Ecart
 * @package     Ecart_Install
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 *
 * @category    Ecart
 * @package     Ecart_Install
 * @subpackage  Controller
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class IndexController extends Zend_Controller_Action
{
    /**
     * Install model
     *
     * @var Ecart_Install_Model_Wizard
     */
    private $_install;
    
    /**
     * Install session
     *
     * @var Zend_Session_Namespace
     */
    private $_session;
    
    public function init()
    {
        $this->initView();
        $layout = Zend_Layout::getMvcInstance();
        
        $this->_session = Zend_Registry::get('session');
        
        $this->view->baseUrl = Zend_Controller_Front::getInstance()
            ->getBaseUrl();
        $this->view->addHelperPath('app' . DIRECTORY_SEPARATOR .
           'views' . DIRECTORY_SEPARATOR . 'helpers',
           'Ecart_View_Helper'
        );
        $this->view->doctype('XHTML1_STRICT');
        
        $layout->setView($this->view)->setOptions(array(
            'layoutPath' => 'app/views/layouts',
            'layout' => 'layout'
        ));
        
        $this->_install = Ecart_Install_Model_Wizard::getInstance();
    }
    
    public function preDispatch()
    {
        if (Ecart::isInstalled() 
            && !isset($this->_session->permit_installation)) {

            if (!headers_sent()) {
                $host  = $_SERVER['HTTP_HOST'];
                $uri   = rtrim(
                    dirname(str_replace('/install', '', $_SERVER['PHP_SELF'])),
                    '/\\'
                );
                header("Location: http://$host$uri/");
            }
            exit();
        }
    }
    
    public function indexAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_LICENSE);
        $this->view->pageTitle = 'License';
        $this->view->license = @file_get_contents('../license.txt');
        $this->render('step-license');
    }
    
    public function stepRequirementsAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_REQUIREMENTS);
        $this->view->pageTitle = 'Requirements';
        $this->view->checks = $this->_install->checkRequirements();
        $isValid = true;
        foreach ($this->view->checks as $item) {
            if (!$item['success']) {
                $isValid = false;
            }
        }
        if (!$isValid) {
            $this->render('step-requirements');
        } else {
            $this->_redirect('index/step-localization');
        }
    }
    
    public function checkRequirementsAction()
    {       
        foreach ($this->_install->checkRequirements() as $item) {
            if (!$item['success']) {
                $this->_redirect('index/step-requirements');
            }
        }
        $this->_redirect('index/step-localization');
    }
    
    public function stepLocalizationAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_LOCALIZATION);
        $this->view->pageTitle = 'Localization';
        $this->render('step-localization');
    }
    
    public function saveLocalizationAction()
    {
        $default_locale = $this->_getParam('default');
        $additional_locales = $this->_getParam('locale');
        $additional_currencies = $this->_getParam('currency');
        
        $this->_session->locale['locale'] = array();
        $this->_session->locale['timezone'] = array();
        $this->_session->locale['currency'] = array();
        
        $this->_session->locale['locale'][$default_locale['locale']] =
            $default_locale['locale'];
        $this->_session->locale['timezone'][$default_locale['timezone']] =
            $default_locale['timezone'];
        $this->_session->locale['currency'][$default_locale['currency']] =
            $default_locale['currency'];
        
        if (is_array($additional_locales)) {
            foreach ($additional_locales as $locale) {
                $this->_session->locale['locale'][$locale] = $locale;
            }
        }
        if (is_array($additional_currencies)) {
            foreach ($additional_currencies as $currency) {
                $this->_session->locale['currency'][$currency] = $currency;
            }
        }
        $this->_redirect('index/step-configuration');
    }
    
    public function stepConfigurationAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_CONFIGURATION);
        $this->_install->initStore();
        $this->view->pageTitle = 'Configuration';
        
        $this->view->db = array(
            'host'     => $this->_session->db_host,
            'username' => $this->_session->db_username,
            'dbname'   => $this->_session->db_dbname,
            'prefix'   => $this->_session->db_prefix
        );
        
        $this->view->store = array(
            'base_url'   => $this->_session->store_baseUrl,
            'use_ssl'    => $this->_session->use_ssl,
            'secure_url' => $this->_session->store_secureUrl,
            'admin_url'  => $this->_session->store_adminUrl,
            'crypt_key'  => $this->_session->store_cryptKey
        );
        
        
        $this->render('step-configuration');
    }
    
    public function saveConfigurationAction()
    {
        $params = $this->_getAllParams();
        if (empty($params['host']) 
            || empty($params['username'])
            || empty($params['database'])
            || empty($params['base_url'])
            || empty($params['admin_url'])) {

            Ecart::message()->addError(__('Fill all required fields please'));
            $this->_redirect('index/step-configuration');
        }
        
        $this->_session->db_host     = $params['host'];
        $this->_session->db_username = $params['username'];
        $this->_session->db_password = $params['password'];
        $this->_session->db_dbname   = $params['database'];
        $this->_session->db_prefix   = $params['prefix'];
        
        $this->_session->use_ssl = (bool)$params['enable_ssl'];
        $this->_session->store_baseUrl   = $params['base_url'];
        $this->_session->store_secureUrl  = $this->_session->use_ssl ?
            $params['secure_url'] : $params['base_url'];
        $this->_session->store_adminUrl = $params['admin_url'];
        $this->_session->store_cryptKey = $params['crypt_key'];
        
        try {
            $this->_install->checkConnection();
        } catch (Exception $e) {
            $this->_session->store_secureUrl = $params['secure_url'];
            Ecart::message()->addError($e->getMessage());
            $this->_redirect('index/step-configuration');
        }
        // this variable clears in step finish action, and checks in preDispatch
        $this->_session->permit_installation = true;  
        $this->_saveConfig();
        $this->_redirect('index/step-user');
    }
    
    public function stepUserAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_USER);
        $this->view->pageTitle = 'Admin User Settings';
        $this->render('step-user');
    }
    
    public function saveUserAction()
    {
        $params = $this->_getAllParams();
        if (empty($params['login'])  || 
            empty($params['password'])  || 
            empty($params['email'])     || 
            empty($params['name']) || 
            empty($params['surname']))
        {
            Ecart::message()->addError(__('Fill all required fields please'));
            $this->_redirect('index/step-user');
        }
        
        $this->_session->user_firstname = $params['name'];
        $this->_session->user_lastname  = $params['surname'];
        $this->_session->user_login     = $params['login'];
        $this->_session->user_email     = $params['email'];
        $this->_session->user_password  = $params['password'];
        $this->_redirect('index/step-modules');
    }
    
    public function stepModulesAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_MODULES);
        $this->view->pageTitle = 'Installation mode';
        $this->view->modules = Ecart_Install_Model_Module::getModules();
        $this->render('step-modules');
    }
    
    public function saveModulesAction()
    {
        $this->_session->modules = array_keys(
            array_filter($this->_getParam('modules'))
        );
        $this->_install();
    }
    
    public function stepFinishAction()
    {
        $this->_install->setStep(Ecart_Install_Model_Wizard::STEP_FINISH);
        $this->view->pageTitle = 'Finish';
        $this->view->user_login = $this->_session->user_login;
        $this->view->user_password = $this->_session->user_password;
        $this->view->crypt_key = $this->_session->store_cryptKey;
        $this->view->admin_route = trim($this->_session->store_adminUrl, '/ ');
        $this->_session->unsetAll();
        $this->render('step-finish');
    }
    
    private function _install()
    {
        try {
            $this->_install
                ->installCore()
                ->run()
                ->applyTemplate();
            
        } catch (Exception $e) {
            Ecart::message()->addError($e->getMessage());
            Ecart::message()->addError($e->getTraceAsString());
            $this->_redirect('index/step-modules');
        }
        $this->_redirect('index/step-finish');
    }

    private function _saveConfig()
    {
        if (!file_exists($this->_getConfigPath())) {
            @touch($this->_getConfigPath());
        }
        if (is_writable($this->_getConfigPath())) {
            $content = $this->_getConfigContent();
            file_put_contents($this->_getConfigPath(), $content);
        } else {
            throw new Zend_Exception(
                __("Config file is not writable at %s", $this->_getConfigPath())
            );
        }
    }
    
    private function _getConfigContent()
    {
        $config = array();
        foreach ($this->_session as $key => $value) {
            $config[$key] = $value;
        }
        if (empty($config['store_cryptKey'])) {
            $config['store_cryptKey'] = md5($this->_install->generateKey());
        }
        
        $this->view->config = $config;
        
        return $this->view->render('index/config.phtml');
    }
    
    private function _getConfigPath()
    {
        return $this->_session->store_path . '/app/etc/config.php';
    }
}
