<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Locale
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Locale
{
    const DEFAULT_LOCALE    = 'en_US';
    const DEFAULT_CURRENCY  = 'USD';
    const DEFAULT_TIMEZONE  = 'America/Los_Angeles';
    
    /**
     * Set locale and language if possible
     *
     * @static
     * @param string (locale or language) $locale
     */
    public static function setLocale($locale = 'auto')
    {
        $nsMain = Ecart::session();
        
        if (!strstr($locale, '_')) {
            $locale = self::_getLocaleFromLanguageCode($locale);
        }
        
        if (Zend_Registry::isRegistered('Zend_Locale')) {
            $currentLocale = Zend_Registry::get('Zend_Locale');
            $currentLocale->setLocale($locale);
        } else {
            try {
                $currentLocale = new Zend_Locale($locale);
            } catch (Zend_Locale_Exception $e) {
                $currentLocale = new Zend_Locale(self::DEFAULT_LOCALE);
            }
            Zend_Locale::setCache(Ecart::cache());
            Zend_Registry::set('Zend_Locale', $currentLocale);
        }
        
        if (Zend_Registry::isRegistered('app') && Zend_Registry::get('app') == 'admin') {
            $nsMain->locale = $locale;
        }
        
        $row = Ecart::single('locale/language')->fetchRow(Ecart::db()->quoteInto(
            'code = ?', $currentLocale->getLanguage()
        ));
        
        if ($row) {
            $nsMain->language = $row->id;
        } else {
            $nsMain->language = Ecart::config()->main->store->language;
        }
        self::setTimezone();
    }
    
    /**
     * Retrieve first suitable locale with language
     *
     * @static
     * @param string $code Language ISO code 
     * @return string Locale ISO code
     */
    private static function _getLocaleFromLanguageCode($code)
    {
        $localeList = self::getLocaleList(true);
        
        foreach ($localeList as $locale) {
            if (strstr($locale, $code)) {
                return $locale;
            }
        }
        
        return self::DEFAULT_LOCALE;
    }
    
    /**
     * Retrieve locale object
     *
     * @static
     * @return Zend_Locale
     */
    public static function getLocale()
    {
        if (!Zend_Registry::isRegistered('Zend_Locale')) {
            if (Zend_Registry::get('app') == 'front' 
                && Ecart_Controller_Router_Route::hasLocaleInUrl()) {

                self::setLocale(Ecart_Controller_Router_Route::getCurrentLocale());
            } elseif (Zend_Registry::get('app') == 'admin' 
                && isset(Ecart::session()->locale)) {

                self::setLocale(Ecart::session()->locale);
            } else {
                self::setLocale(Ecart::config()->main->store->locale);
            }
        }
        return Zend_Registry::get('Zend_Locale');
    }
    
    /**
     * Retrieve default locale from config
     *
     * @static
     * @return string Locale ISO code
     */
    public static function getDefaultLocale()
    {   
        return empty(Ecart::config()->main->store->locale) ? 
           self::DEFAULT_LOCALE : Ecart::config()->main->store->locale;
    }
    
    /**
     * Returns a list of all known locales, or all installed locales
     *
     * @static
     * @return array
     */
    public static function getLocaleList($installedOnly = false)
    {
        if (!$installedOnly) {
            return array_keys(Zend_Locale::getLocaleList());
        }
        return Ecart::single('locale/language')->getLocaleList();
    }
    
    /**
     * Retrieve languageId from session;
     *
     * @static
     * @return int
     */
    public static function getLanguageId()
    {
        if (!isset(Ecart::session()->language)) {
            Ecart::session()->language = Ecart::config()->main->store->language;
        }
        return Ecart::session()->language;
    }
    
    /**
     * Retrieve part of url, responsible for locale
     *
     * @static
     * @return string Part of url ('/uk')
     */
    public static function getLanguageUrl()
    {
        
        $language = self::getLocale()->getLanguage();
        $locale = self::getLocale()->toString();
        
        if ($locale == self::getDefaultLocale()) {
            return '';
        }
        if ($locale == self::_getLocaleFromLanguageCode($language)) {
            return '/' . $language;
        }
        
        return '/' . $locale;
    }
    
    /**
     * get default store timezone
     *
     * @static
     * @return  string  example : "Australia/Hobart"
     */
    public static function getDefaultTimezone()
    {
        return Ecart::config()->main->store->timezone;
    }
    
    /**
     * get timezone
     *
     * @static
     * @return string  
     */
    public static function getTimezone()
    {
        return date_default_timezone_get();
    }
    
    /** 
     * set timezone
     *
     * @static
     * @param mixed void|string
     * @return bool  
     */
    public static function setTimezone($timezone = null)
    {
        if (null === $timezone) {
            $timezone = Ecart::config()->main->store->timezone;
        }
        if (@date_default_timezone_set($timezone)) {
            return true;
        }
        return @date_default_timezone_set(self::DEFAULT_TIMEZONE);
    }
    
    /**
     * Retrieve the list of available admin intrerface tranlations
     *
     * @static
     * @return array
     */
    public static function getAdminLocales()
    {
        if (!$locales = Ecart::cache()->load('locales_list')) {
            $path = Ecart::config()->system->path . '/app/locale';
            
            try {
                $locales_dir = new DirectoryIterator($path);
            } catch (Exception $e) {
                throw new Ecart_Exception("Directory $path not readable");
            }
            
            $locale = Ecart_Locale::getLocale();
            $locales = array();
            
            foreach ($locales_dir as $localeDir) {
                $localeCode = $localeDir->getFilename();
                if ($localeCode[0] == '.' || !strstr($localeCode, '_')) {
                    continue;
                }
                list($language, $country) = explode('_', $localeCode, 2);
                
                $language = $locale->getTranslation($language, 'language', $locale);
                $country = $locale->getTranslation($country, 'country', $locale);
                if (!$language) {
                    $language = $locale->getTranslation(
                        $language, 'language', Ecart_Locale::DEFAULT_LOCALE
                    );
                }
                if (!$country) {
                    $country = $locale->getTranslation(
                        $country, 'country', Ecart_Locale::DEFAULT_LOCALE
                    );
                }
                $locales[$localeCode] = ucfirst($language) . ' (' . $country . ')';
            }
            ksort($locales);
            Ecart::cache()->save($locales, 'locales_list', array('locales'));
        }
        return $locales;
    }
}