<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Shipping
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Shipping
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Shipping
{
    /**
     * @static
     * @var array
     */
    private static $_methods;
    
    /**
     * Retrieve the list of shipping methods of installed shipping modules
     *
     * @static
     * @return array
     */
    public static function getMethodNames()
    {
        if ($methods = Ecart::cache()->load('shipping_methods_list')) {
            return $methods;
        }
        
        $prefix = 'Shipping';
        $modules = Ecart::getModules();
        $methods = array();
        foreach($modules as $path) {
            $moduleName = str_replace(
                Ecart::config()->system->path . '/app/code/Ecart/', '', $path
            );
            if (substr($moduleName, 0, strlen($prefix)) != $prefix) {
                continue;
            }
            $dir = opendir($path . '/Model');
            while ($fname = readdir($dir)) {
                if (!is_file("{$path}/Model/{$fname}")) {
                    continue;
                }
                list($methodName, $ext) = explode('.', $fname, 2);
                if ($ext != 'php') {
                    continue;
                }
                $methods[] = substr($moduleName, strlen($prefix)) . '_' . $methodName;
            }
            closedir($dir);
        }
        Ecart::cache()->save(
            $methods, 'shipping_methods_list', array('modules')
        );
        return $methods;
    }
    
    /**
     * Retrieve array of installed shipping methods
     *
     * @static
     * @return array
     */
    public static function getMethods()
    {
        $methods = array();
        //Zend_Debug::dump(self::getMethodNames()); exit();
        foreach (self::getMethodNames() as $name) {
            $method = self::getMethod($name);
            $methods[$method->getCode()] = $method;
        }
        return $methods;
    }
    
    /**
     * Retrieve shipping method class
     *
     * @param string $code
     * @return Ecart_Method_Shipping_Model_Abstract|null
     */
    public static function getMethod($code)
    {
        if (isset(self::$_methods[$code])) {
            return self::$_methods[$code];
        }
        
        list($moduleName, $methodName) = explode('_', $code, 2);
        
        $methodType = null;
        if (strstr($methodName, '_')) {
            list($methodName, $methodType) = explode('_', $methodName, 2);
        }
        $className = 'Ecart_Shipping' . $moduleName . '_Model_' . $methodName;
        self::$_methods[$code] = new $className($methodType);
        return self::$_methods[$code];
    }
    
    /**
     * Retrieve the list of allowed methods,
     * according to checkout process request
     *
     * @static
     * @return array('methods' => array(), 'currentMethodCode' => string)
     */
    public static function getAllowedMethods()
    {
        if (!function_exists('_sortOrder')) {
            function _sortOrder($a, $b)
            {
                if ($a['sortOrder'] == $b['sortOrder']) {
                    return 0;
                }
                return $a['sortOrder'] < $b['sortOrder'] ? -1 : 1;
            }
        }
        $checkout =  Ecart::single('checkout/checkout');
        $request = $checkout->getShippingRequest();
        $currentMethodCode = null;
        $methods = array();

        foreach (self::getMethods() as $method) {
            if (!$method->isEnabled() || !$method->isAllowed($request)) {
                continue;
            }
            foreach ($method->getTypes($request) as $type) {
                if ($checkout->getStorage()->shipping == $type['id'] 
                    || !count($methods)) {

                    $currentMethodCode = $type['id'];
                }
                $methods[$method->getCode()][] = $type;
            }
            $methods[$method->getCode()]['sortOrder'] = $method->config()->sortOrder;
        }
        uasort($methods, '_sortOrder');

        foreach ($methods as &$method) {
            unset($method['sortOrder']);
        }
        
        return array (
            'methods' => $methods, 
            'currentMethodCode' => $currentMethodCode
        );
    }
}